#' @name dafilter.default
#' @aliases dafilter.default
#' @title Implement the default filter which is the Kalman filter
#' @description This function implements the Kalman filter for a given set of model inputs and observations. The Kalman filter is the optimal estimator if the model is linear and the measurement/ process noises are normally distributed.
#' @param model_handle               The function that executes the model, must take in a numvarsx1 input and return a numvarsx1 output, where numvars is the number of inputs to the model
#' @param measurement_handle         The function that converts the model output state to measurement
#' @param observations               The set of observations that the filter will use to update its estimates.
#' @param input                      A list with: the first guess of forecast error covariance matrix (Pguess),  first guess values of forecast vector (xguess), model process error covariance matrix (Qmod), measurement covaraince (R), and, time resolution of model (T)
#' @param option                     Either: "KALMAN", "EXTENDED", "ENSEMBLE", "PARTICLE"
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  A filter class which is a list comprising of the mean of the forecasted variables and their associated covariance matrix
#' @references %% ~put references to the literature/web site here ~
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso  \code{\link{kalmanfilter}} \code{\link{extendedkalmanfilter}} \code{\link{ensemblekalmanfilter}} \code{\link{particlefilter}}
#' @examples
#' Example 1: running the demonstration example
#' model_handle       <- f_model
#' measurement_handle <- h_meas
#' input              <- setdefault()
#' setup              <- problemsetup(input)
#' kalmanout          <- dafilter(model_handle, measurement_handle, setup$z, input, option="KALMAN")
#' Example 2: running the lake model
#' model_handle       <- f_lake
#' measurement_handle <- h_lake
#' input              <- setlakedefault()
#' setup              <- lakesetup(input)
#' ensembleout        <- dafilter(model_handle, measurement_handle, setup$z, input, option="ENSEMBLE")
#' @importFrom numDeriv jacobian
#' @export

dafilter.default <-
  function(model_handle, measurement_handle,  observations, input, option = "KALMAN")
  {
    if (option == "KALMAN") {
      filterout          <- kalmanfilter(model_handle, measurement_handle, setup$z, input)
    } else if (option == "EXTENDED") {
      filterout          <- extendedkalmanfilter(model_handle, measurement_handle, setup$z, input)
    } else if (option == "ENSEMBLE")  {
      filterout          <- ensemblekalmanfilter(model_handle, measurement_handle, setup$z, input)
    } else if (option == "PARTICLE") {
      filterout          <- particlefilter(model_handle, measurement_handle, setup$z, input)
    }
  }