#' @name ensemblekalmanfilter
#' @aliases ensemblekalmanfilter
#' @title Implement The Ensemble Kalman Filter
#' @description This function implements the ensemble Kalman filter for a given set of model inputs and observations. The filter approximates a Gaussian distribution using a set of randomly generated, and is a good choice for models with significant non linearity.
#' @param input A list with the problem setup including the first guess of forecast error covariance matrix (Pguess),  first guess values of forecast vector (xguess), model process error covariance matrix (Qmod), measurement covaraince (R), and, time step of model (timestep)
#' @param observations The set of observations that the filter will use to update its estimates.
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  A filter class which is a list comprising of the mean of the forecasted variables and their associated covariance matrix
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso  \code{\link{kalmanfilter}}, \code{\link{extendedkalmanfilter}}, \code{\link{particlefilter}}
#' @examples
#' input              <- setdefault()
#' setup              <- problemsetup(input)
#' ensembleout        <- ensemblekalmanfilter(input,setup$z)
#' @importFrom numDeriv jacobian
#' @export

ensemblekalmanfilter <-
  function(input, observations)
  {
    # starting time counter
    start.time <- Sys.time()
    # assign function handles
    model_handle         <- input$model_handle
    measurement_handle   <- input$measurement_handle
    param_handle         <- input$param_handle
    parammeas_handle     <- input$parammeas_handle
    # assign variables
    xguess               <- input$xguess
    Pxguess              <- input$Pxguess
    xguesspars           <- input$xguesspars
    Pxguesspars          <- input$Pxguesspars
    Qmod                 <- input$Qmod
    R                    <- input$R
    timestep             <- input$timestep
    Bmat                 <- input$Bmat
    uinput               <- input$uinput
    numpars              <- input$numpars
    numstates            <- input$numstates
    n_ens                <- input$n_ens
    N                    <- length(seq(input$tbeg, input$tend, timestep))       
    observations         <- as.matrix(observations)
    Pxguess              <- as.matrix(Pxguess)
    xguess               <- as.numeric(xguess)
    Pxguesspars          <- as.matrix(Pxguesspars)
    xguesspars           <- as.numeric(xguesspars)
    n_ens                <- as.numeric(n_ens)
    # estimate the process noise matrix
    Qstates              <- getQ(Qmod[1:numstates], numstates)
    numbeg               <- numstates+1
    numvars              <- numpars+numstates
    Qpars                <- getQ(Qmod[numbeg:numvars], numpars)
    # number of variables that can be measured, and hence forecasted
    numfor               <- length(measurement_handle(array(xguess,dim=c(numstates,1))))    
    # intialize some variables
    # Kalman gain
    Kstate               <- array(NA, dim=c(numstates,N))    
    Kpars                <- array(NA, dim=c(numpars,N))    
    # store the mean of the updated ensemble for all time steps
    x_ens_mean           <- array(NA, dim=c(numvars,N))
    # store the ensemble members at current time step
    x_ens                <- array(NA, dim=c(numstates,n_ens))
    x_enstemp            <- array(NA, dim=c(numstates,n_ens))
    x_enspars            <- array(NA, dim=c(numpars,n_ens))
    # store the analysis mean, i.e., the mean of the updated ensemble for all time steps
    x_ana_mean           <- array(NA, dim=c(numvars,N))
    # error covariance matrix after update for all time steps
    P_analysis           <- array(NA, dim=c(numvars,numvars,N))
    # store the forecast mean, i.e., the mean of the forecasted ensemble for all time steps
    x_for_mean           <- array(NA, dim=c(numstates,N))
    # error covariance matrix before update for all time steps
    P_forecast           <- array(NA, dim=c(numvars,numvars,N))
    # number of variables that can be measured, and hence forecasted
    numfor               <- length(measurement_handle(x_ens[,1]))
    # store the forecasts for all forecasted variables, and ensembles at current time step
    xfor                 <- array(NA, dim=c(numfor,n_ens))          
    # store all ensembles across all time periods
    x_enkfall            <- array(list(), N) 
    # store the deviation of each updated ensemble member from the analysis mean at current time step
    x_ana_pert           <- array(NA, dim=c(numvars,n_ens))
    # store the perturbation of the ensemble (deviation from the mean of each ensemble) at current time step
    x_ens_pert           <- array(NA, dim=c(numstates,n_ens))
    x_ens_pertpars       <- array(NA, dim=c(numpars,n_ens))
    x_for_pert           <- array(NA, dim=c(numstates,n_ens))    
    # generating the parameter ensemble by sampling a multivariate distribution
    if (input$prior == 0){
      # generate random samples using intial covariance estimate Pguess
      temp                <- multirandnorm(xguesspars,Pxguesspars,n_ens,input$seed[2])
      # assign
      x_enspars           <- t(temp)
      rm(temp)
    } else {
      location            <- rep(NA, numpars)
      scale               <- rep(NA, numpars)
      for (i in 1:numpars) {
        tempmean          <- xguesspars[i]
        tempvar           <- Pxguesspars[i,i]
        location[i]       <- log(tempmean/sqrt(1+tempvar/tempmean^2))
        scale[i]          <- (log(1+tempvar/tempmean^2))
      }
      if(length(scale)==1) {
        scale             <- scale
      } else {
        scale             <- diag(scale)
      }
      # generate random multi-variate noise
      temp                <- multirandnorm(location, scale, n_ens, input$seed[2])
      # estimate the ensemble
      x_enspars           <- t(exp(temp))
      rm(temp)
    }
    rm(location, scale)
    # generating the state ensemble by sampling a multivariate distribution
    if (input$prior == 0){
      # generate random samples using intial covariance estimate Pguess
      temp                <- multirandnorm(xguess,Pxguess,n_ens,input$seed[2])
      # assign
      x_ens               <- t(temp)
      rm(temp)
    } else {
      location            <- rep(NA, numstates)
      scale               <- rep(NA, numstates)
      for (i in 1:numstates) {
        tempmean          <- xguess[i]
        tempvar           <- Pxguess[i,i]
        location[i]       <- log(tempmean/sqrt(1+tempvar/tempmean^2))
        scale[i]          <- (log(1+tempvar/tempmean^2))
      }
      if(length(scale)==1) {
        scale             <- scale
      } else {
        scale             <- diag(scale)
      }
      # generate random multi-variate noise
      temp                <- multirandnorm(location,scale,n_ens,input$seed[2])        
      # estimate the final ensemble
      x_ens               <- t(exp(temp))
      rm(temp)
    }
    if (input$name=='lakemodel') {
      # remove unrealistic parameter estimates
      temp1 <- lakeparscheck(x_enspars)
      # remove unrealistic state estimates
      temp2 <- lakestatecheck(x_ens)
      temp  <- union(temp1,temp2)
    } else {
      temp <- c()
    }
    if (length(temp)>0) {
      x_enspars           <- x_enspars[,-temp]        
      n_ens               <- dim(x_enspars)[2]       
      x_ens               <- array(x_ens[,-temp],c(numstates,n_ens))  
      x_enstemp           <- array(x_enstemp[,-temp],c(numstates,n_ens))  
      # store the forecasts for all forecasted variables, and ensembles at current time step
      xfor                <- array(NA, dim=c(numfor,n_ens))          
      # store the deviation of each updated ensemble member from the analysis mean at current time step
      x_ana_pert          <- array(NA, dim=c(numvars,n_ens))
      # store the perturbation of the ensemble (deviation from the mean of each ensemble) at current time step
      x_ens_pert          <- array(NA, dim=c(numstates,n_ens))
      x_ens_pertpars      <- array(NA, dim=c(numpars,n_ens))
      x_for_pert          <- array(NA, dim=c(numstates,n_ens))
      rm(location, scale,temp1,temp2,temp)
    }
    if (is.null(n_ens)  || n_ens==0) {
      x_ens_mean[,time]  <- NA
      x_for_mean[,time]  <- NA
      Kstate[,time]      <- NA
      Kpars[,time]       <- NA
      x_ana_mean[,time]  <- NA
      P_forecast[,,time] <- NA
      x_enkfall[[time]]  <- NA
      P_analysis[,,time] <- NA
    } else {
      # Now loop through time, and ensembles, iterating over dynamics-propagate / meas-update...
      for (time in 1:N)  { 
        # Step 1.Predict using the parameter ensemble
        # State update (also generates the state ensemble for t=1)      
        for (ens in 1:n_ens)
        { # run the model
          x_enstemp[,ens]   <- addprocessnoise(model_handle, Qstates, input$qtype,  ens+n_ens*time, x_ens[,ens], x_enspars[,ens], timestep, time) + Bmat*uinput[time]                
        }              
        # convert prediction to measurement forecast
        for (ens in 1:n_ens)
        {      
          xfor[,ens]        <- measurement_handle(x_enstemp[,ens])
        } # end loop over ensembles
        # Step 2. Update the parameter ensemble
        # first, estimate mean of the predicted ensemble from previous time step
        x_ens_mean[1:numstates,time]     <- apply(x_enstemp, 1, mean)
        x_ens_mean[numbeg:numvars,time]  <- apply(x_enspars, 1, mean)
        # store the forecast from previous time step
        x_for_mean[,time]                <- apply(xfor, 1, mean)
        # then, estimate the deviation from the mean for each member of the ensemble
        x_ens_pertpar                    <- x_enspars[1:numpars,]  - x_ens_mean[numbeg:numvars,time]
        x_for_pert                       <- xfor       - x_for_mean[,time]      
        # estimate the covariance of the prediction ensemble
        Pf                               <- x_for_pert%*%t(x_for_pert)/(n_ens-1)
        # estimate the cross covariance between the parameter ensemble and the prediction ensemble
        Pfparpred                        <- x_ens_pertpar%*%t(x_for_pert)/(n_ens-1)
        # generate noise to add on top of observation at time i
        # set seed
        set.seed(input$seed[2]+time)
        # generate noise
        meas_noise                       <- t(sqrt(input$R)*rnorm(n = n_ens, mean = 0, sd = 1))
        # approximate R
        approx_R                         <- (meas_noise%*%t(meas_noise))/(n_ens-1)
        # now start updating the parameters, first estimate the Kalman gain
        Kpars[,time]                     <- Pfparpred%*%solve(Pf+approx_R)    
        # updated estimates
        for (i in 1:numpars) {
          x_enspars[i,]                  <- x_enspars[i,]+Kpars[i,time]%*%(rep(observations[time],length=n_ens)+meas_noise-xfor[1:numstates,])      
        }
        # remove unrealisitic parameter estimates
        if (input$name=='lakemodel') {
          temp  <- lakeparscheck(x_enspars)
        } else {
          temp  <- c()
        }
        if (length(temp)>0) {
          x_enspars             <- x_enspars[,-temp]        
          n_ens                 <- dim(x_enspars)[2]  
          meas_noise            <- array(meas_noise[,-temp],c(1,n_ens)) 
          x_ens                 <- array(x_ens[,-temp],c(numstates,n_ens)) 
          # store the forecasts for all forecasted variables, and ensembles at current time step
          xfor                  <- array(NA, dim=c(numfor,n_ens))          
          # store the deviation of each updated ensemble member from the analysis mean at current time step
          x_ana_pert            <- array(NA, dim=c(numvars,n_ens))
          # store the perturbation of the ensemble (deviation from the mean of each ensemble) at current time step
          x_ens_pert            <- array(NA, dim=c(numstates,n_ens))
          x_ens_pertpars        <- array(NA, dim=c(numpars,n_ens))
          x_for_pert            <- array(NA, dim=c(numstates,n_ens))
          rm(temp)
        }  
        if (is.null(n_ens) || n_ens==0) {
          x_ens_mean[,time]  <- NA
          x_for_mean[,time]  <- NA
          Kstate[,time]      <- NA
          Kpars[,time]       <- NA
          x_ana_mean[,time]  <- NA
          P_forecast[,,time] <- NA
          x_enkfall[[time]]  <- NA
          P_analysis[,,time] <- NA
          break;
        }
        # Step 3. Predict using the updated parameter ensemble
        for (ens in 1:n_ens)
        { # run the model
          x_ens[,ens]                    <- addprocessnoise(model_handle, Qstates, input$qtype,  ens+n_ens*time, x_ens[,ens], x_enspars[,ens], timestep, time) + Bmat*uinput[time]        
        }  
        # convert prediction to measurement forecast
        for (ens in 1:n_ens)
        {      
          xfor[,ens]                     <- measurement_handle(x_ens[,ens])
        } # end loop over ensembles    
        # Step 4. Update the state 
        # first, estimate mean of the predicted ensemble from previous time step
        x_ens_mean[1:numstates,time]     <- apply(x_ens, 1, mean)
        x_ens_mean[numbeg:numvars,time]  <- apply(x_enspars, 1, mean)
        # store the forecast from previous time step
        x_for_mean[,time]                <- apply(xfor, 1, mean)
        # then, estimate the deviation from the mean for each member of the ensemble
        x_ens_pert                       <- x_ens  - x_ens_mean[1:numstates,time]
        x_for_pert                       <- xfor   - x_for_mean[,time]      
        # estimate the covariance of the prediction ensemble
        Pf                               <- x_for_pert%*%t(x_for_pert)/(n_ens-1)
        # estimate the cross covariance between the state ensemble and the prediction ensemble
        Pfstatepred                      <- x_ens_pert%*%t(x_for_pert)/(n_ens-1)
        # estimate the Kalman gain for the state
        Kstate[,time]                    <- Pfstatepred%*%solve(Pf+approx_R)    
        # updated estimates
        x_ens                            <- x_ens+Kstate[,time]%*%(rep(observations[time],length=n_ens)+meas_noise-xfor)      
        # check the updated states, if they fall below zero, remove them
        # remove unrealistic state estimates
        if (input$name=='lakemodel') {
          temp <- lakestatecheck(x_ens)
        } else {
          temp <- c()
        }
        if (length(temp)>0) {
          x_enspars             <- x_enspars[,-temp]    
          n_ens                 <- dim(x_enspars)[2]       
          x_ens                 <- array(x_ens[,-temp],c(numstates,n_ens)) 
          # store the forecasts for all forecasted variables, and ensembles at current time step
          xfor                  <- array(NA, dim=c(numfor,n_ens))          
          # store the deviation of each updated ensemble member from the analysis mean at current time step
          x_ana_pert            <- array(NA, dim=c(numvars,n_ens))
          # store the perturbation of the ensemble (deviation from the mean of each ensemble) at current time step
          x_ens_pert            <- array(NA, dim=c(numstates,n_ens))
          x_ens_pertpars        <- array(NA, dim=c(numpars,n_ens))
          x_for_pert            <- array(NA, dim=c(numstates,n_ens))
          rm(temp)
        }           
        if (is.null(n_ens)  || n_ens==0) {
          x_ens_mean[,time]  <- NA
          x_for_mean[,time]  <- NA
          Kstate[,time]      <- NA
          Kpars[,time]       <- NA
          x_ana_mean[,time]  <- NA
          P_forecast[,,time] <- NA
          x_enkfall[[time]]  <- NA
          P_analysis[,,time] <- NA
          break;
        }
        # calculate the analysis mean, i.e., the mean of the updated ensemble
        x_ana_mean[1:numstates,time]     <- array(apply(x_ens, 1, mean),dim=c(numstates,1))
        x_ana_mean[numbeg:numvars,time]  <- array(apply(x_enspars, 1, mean),dim=c(numpars,1))
        # estimate deviation from the analysis mean for each member of the updated ensemble 
        x_ana_pert[1:numstates,]         <- x_ens  - x_ana_mean[1:numstates,time]
        for (i in 1:numpars) {
          j                              <- i+1
          x_ana_pert[j,]                 <- x_enspars[i,]  - x_ana_mean[j,time]
        }    
        # P_analysis is the analysis error covariance matrix
        P_analysis[,,time]               <-  x_ana_pert%*%t(x_ana_pert)/(n_ens-1)      
        # save Pf in forecast variance estimate
        P_forecast[,,time]               <- Pf      
        x_enkfall[[time]]                <- array(c(t(x_ens), t(x_enspars)),dim=c(n_ens,numvars)) 
        #clean up
        rm(Pf, Pfparpred, Pfstatepred, meas_noise, approx_R)
      }
    }
    rm(x_ens_mean, x_ens, x_enspars, x_ens_pert, x_ens_pertpars, xfor, x_for_pert, x_ana_pert)
    Kgain            <- array(c(t(Kstate), t(Kpars)),dim=c(N,numvars))    
    #return output
    filterest        <- list(name='EnsembleKalman', x_mean=x_ana_mean, x_cov=P_analysis, x_for_mean=x_for_mean, x_for_cov=P_forecast, Kgain=Kgain,x_enkfall=x_enkfall)
    filterest$call   <- match.call()
    #define the fitler class
    class(filterest) <- "filter"
    # ending time counter
    end.time   <- Sys.time()
    filterest$timetaken <- end.time - start.time
    if (input$timeflag == 1) {print(paste("Function time is " , round(as.numeric(filterest$timetaken, units="secs"),2), "seconds"))}    
    filterest
  }