#' @name extendedkalmanfilter
#' @aliases extendedkalmanfilter
#' @title Implement the extended Kalman filter
#' @description This function implements the extended Kalman filter for a given set of model inputs and observations. 
#' @param input A list with the problem setup including the first guess of forecast error covariance matrix (Pguess),  first guess values of forecast vector (xguess), model process error covariance matrix (Qmod), measurement covaraince (R), and, time step of model (timestep)
#' @param observations The set of observations that the filter will use to update its estimates.
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  A filter class which is a list comprising of the mean of the forecasted variables and their associated covariance matrix
#' @references %% ~put references to the literature/web site here ~
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso  \code{\link{kalmanfilter}}, \code{\link{ensemblekalmanfilter}}, \code{\link{particlefilter}}
#' @examples
#' Example: running the lake model
#' input              <- setlakedefault(1) # 1 for the eutrophication case
#' setup              <- lakesetup(input)
#' exkalout           <- extendedkalmanfilter(input,setup$z)
#' @importFrom numDeriv jacobian
#' @export

extendedkalmanfilter <-
  function(input,observations)
  {
    # starting time counter
    start.time            <- Sys.time()  
    # assign function handles
    model_handle          <- input$model_handle
    measurement_handle    <- input$measurement_handle
    param_handle          <- input$param_handle
    parammeas_handle      <- input$parammeas_handle
    # assign variables
    xguess                <- input$xguess
    Pxguess               <- input$Pxguess
    xguesspars            <- input$xguesspars
    Pxguesspars           <- input$Pxguesspars
    Qmod                  <- input$Qmod
    R                     <- input$R
    timestep              <- input$timestep
    Bmat                  <- input$Bmat
    uinput                <- input$uinput
    numpars               <- input$numpars
    numstates             <- input$numstates
    N                     <- length(seq(input$tbeg, input$tend, timestep))       
    observations          <- as.matrix(observations)
    Pxguess               <- as.matrix(Pxguess)
    xguess                <- as.numeric(xguess)
    Pxguesspars           <- as.matrix(Pxguesspars)
    xguesspars            <- as.numeric(xguesspars)
    # estimate the process noise matrix
    Qstates               <- getQ(Qmod[1:numstates], numstates)
    numbeg                <- numstates+1
    numend                <- numpars+numstates
    Qpars                 <- getQ(Qmod[numbeg:numend], numpars)
    # number of variables that can be measured, and hence forecasted
    numfor                <- length(measurement_handle(array(xguess,dim=c(numstates,1))))
    # intialize some variables
    Fmatstate             <- array(NA, dim=c(numstates,numstates))
    Hmatstate             <- array(NA, dim=c(numstates,numfor))    
    Kstate                <- array(NA, dim=c(numstates,N))
    Fmatpars              <- array(NA, dim=c(numpars,numpars))
    Hmatpars              <- array(NA, dim=c(numpars,numfor))    
    Kpars                 <- array(NA, dim=c(numpars,N))
    x_for_mean            <- array(NA, dim=c(numstates,N))
    P_for_mean            <- array(NA, dim=c(numstates,numstates,N))     
    P_kfstate             <- array(NA, dim=c(numstates,numstates,N))
    x_kfstate             <- array(NA, dim=c(numstates,N))
    Pminusstate           <- array(NA, dim=c(numstates,numstates))
    xminusstate           <- array(NA, dim=c(numstates,1))
    x_forstate            <- array(NA, dim=c(numstates,N))
    P_forstate            <- array(NA, dim=c(numstates,numstates,N))        
    P_kfpars              <- array(NA, dim=c(numpars,numpars,N))
    x_kfpars              <- array(NA, dim=c(numpars,N))
    Pminuspars            <- array(NA, dim=c(numpars,numpars))
    xminuspars            <- array(NA, dim=c(numpars,1))
    x_forpars             <- array(NA, dim=c(numpars,N))
    P_forpars             <- array(NA, dim=c(numpars,numpars,N))    
    
    # now loop, iterating over dynamics- propagate / measurement update
    for (i in 1:N) {      
      if (i==1) {
        # the extended Kalman filter approximates the non linear model with the derivative at each step
        Fmatstate          <- jacobian(model_handle,       xguess, method = "Richardson", theta=xguesspars)
        Hmatstate          <- jacobian(measurement_handle, xguess, method = "Richardson")
        Fmatpar            <- jacobian(param_handle,       xguesspars, method = "Richardson")
        Hmatpar            <- jacobian(parammeas_handle,   xguesspars, method = "Richardson", state=xguess)        
        # parameter prediction
        xminuspars         <- xguesspars
        Pminuspars         <- Pxguesspars + Qpars
        # state prediction
        xminusstate        <- Fmatstate%*%xguess  + Bmat%*%uinput[i] 
        Pminusstate        <- Fmatstate%*%Pxguess%*%t(Fmatstate) + Qstates
      } else {
        # the extended Kalman filter approximates the non linear model with the derivative at each step        
        Fmatstate          <- jacobian(model_handle,       x_kfstate[,i-1], method = "Richardson", theta=x_kfpars[,i-1])
        Hmatstate          <- jacobian(measurement_handle, x_kfstate[,i-1], method = "Richardson")
        Fmatpar            <- jacobian(param_handle,       x_kfpars[,i-1],  method = "Richardson")
        Hmatpar            <- jacobian(parammeas_handle,   x_kfpars[,i-1],  method = "Richardson", state=x_kfstate[,i-1])                
        # parameter prediction
        xminuspars         <- x_kfpars[,i-1]
        Pminuspars         <- P_kfpars[,,i-1] + Qpars
        # state prediction
        xminusstate        <- Fmatstate%*%x_kfstate[,i-1] + Bmat%*%uinput[i] 
        Pminusstate        <- Fmatstate%*%P_kfstate[,,i-1]%*%t(Fmatstate) + Qstates
      }
      # set negative parameters to 0
      for (par in 1:numpars) {
        # no parameter can be negative
        if ((is.na(xminuspars[par])==FALSE) || (is.nan(xminuspars[par])==FALSE)) {
          if (xminuspars[par]<0) {
            xminuspars[par]   <- 0
          }
          # b parameter cannot be greater than 1
          if (par==1) {
            if (xminuspars[par]>1) {
              xminuspars[par]  <- 1       
            }
          } else {
            if (xminuspars[par]<2) {
              xminuspars[par]  <- 2       
            } else if (xminuspars[par]>8) {
              xminuspars[par]  <- 8       
            }
          }
        }
      }
      for (st in 1:numstates) {
        # no state can be negative
        if (is.na(xminusstate[st])==FALSE || is.nan(xminusstate[st])==FALSE) {
          if (xminusstate[st]<0) {
            xminusstate[st] <- 0.05
          } 
        }
      }
      x_for_mean[,i]       <- xminusstate
      P_for_mean[,,i]      <- Pminusstate
      # parameter correction
      Kpars[,i]            <- Pminuspars%*%t(Hmatpar)%*%solve(Hmatpar%*%Pminuspars%*%t(Hmatpar)+R)    
      x_kfpars[,i]         <- xminuspars + Kpars[,i]%*%(observations[i]-Hmatstate%*%xminusstate)
      P_kfpars[,,i]        <- (diag(dim(Pminuspars)[1])-Kpars[,i]%*%Hmatpar)%*%Pminuspars      
      # state correction
      Kstate[,i]           <- Pminusstate%*%t(Hmatstate)%*%solve(Hmatstate%*%Pminusstate%*%t(Hmatstate)+R)
      x_kfstate[,i]        <- xminusstate + Kstate[,i]%*%(observations[i]-Hmatstate%*%xminusstate)
      P_kfstate[,,i]       <- (diag(dim(Pminusstate)[1])-Kstate[,i]%*%Hmatstate)%*%Pminusstate
      # set negative parameters to 0
      for (par in 1:numpars) {
        if ((is.na(x_kfpars[par,i])==FALSE) || (is.nan(x_kfpars[par,i])==FALSE)) {
          if (x_kfpars[par,i]<0) {
            x_kfpars[par,i]   <- 0
          }
          # b parameter cannot be greater than 1
          if (par==1) {
            if (x_kfpars[par,i]>1) {
              x_kfpars[par,i]  <- 1       
            }
          } else {
            if (x_kfpars[par,i]<2) {
              x_kfpars[par,i]  <- 2       
            } else if (x_kfpars[par,i]>8) {
              x_kfpars[par,i]  <- 8       
            }
          }
        }
      }
      for (st in 1:numstates) {
        # no state can be negative
        if ((is.na(x_kfstate[st,i])==FALSE) || (is.nan(x_kfstate[st,i])==FALSE)) {
          if (x_kfstate[st,i]<0) {
            x_kfstate[st,i] <- 0
          }
        }
      }
    } # end loop over time  
    # clear variables
    rm(xminuspars,Pminuspars,xminusstate,Pminusstate)
    # assign output
    x_mean           <- array(c(t(x_kfstate), t(x_kfpars)),dim=c(N,numstates+numpars))
    Kgain            <- array(c(t(Kstate), t(Kpars)),dim=c(N,numstates+numpars))
    # return output
    filterest        <- list(name ='Extended Kalman',x_mean=t(x_mean),x_covstate=P_kfstate,x_covpars=P_kfpars,x_for_mean=x_for_mean,P_for_mean=P_for_mean,Kgain=Kgain)
    filterest$call   <- match.call()
    # define the fitler class
    class(filterest) <- "filter"
    
    # ending time counter
    end.time   <- Sys.time()
    filterest$timetaken <- end.time - start.time
    if (input$timeflag == 1) {print(paste("Function time is " , round(as.numeric(filterest$timetaken, units="secs"),2), "seconds"))}
    
    filterest
  }