#' @name filterdemo
#' @aliases filterdemo 
#' @title Function to run the demonstration example
#' @description This function can be called to run the demostration example that implements the filters. The function allows the user to run the model without any inputs (by setting all values at pre-defined default). Alternatively, the user can enter the choice of filters, random seeds, etc.
#' @param x a vector of inputs to the model
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  %% ~~does not return anything ~~
#' @references Analytic Sciences Corporation (1974), Applied optimal estimation, Gelb, A. (ed.), M.I.T Press, Cambridge, Mass, ISBN: 0262200279
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso  \code{\link{setdefault}}, \code{\link{readinput}}, \code{\link{problemsetup}}, \code{\link{f_model}}, \code{\link{h_meas}}
#' @examples  
#' filterdemo()
#' @export

filterdemo <-
  function() 
  {
    # Mise en place
    rm(list = ls())
    graphics.off()
    
    #########################################################
    ### A) Installing and loading required packages
    #########################################################
    if (!requireNamespace("MASS", quietly = TRUE)) {
      stop("MASS needed for this function to work. Please install it.",
           call. = FALSE)
    }
    
    if (!requireNamespace("mvtnorm", quietly = TRUE)) {
      stop("mvtnorm needed for this function to work. Please install it.",
           call. = FALSE)
    }
    
    if (!requireNamespace("numDeriv", quietly = TRUE)) {
      stop("numDeriv needed for this function to work. Please install it.",
           call. = FALSE)
    }
    
    ########################################################
    #########################################################
    ### B) Input user changeable stuff 
    #########################################################
    # take user's inputs on problem and measurement type, the filters to run
    # and the measurement variance.
    # the user can choose default options to go through the demo
    choice = readline(prompt="Enter whether you would like to enter variables manually or run the default caes: Manual(1) Default(0)  ")
    if (choice == 1) {
      input      <- readinput()
    } else { 
      input      <- setdefault()
    }
    #########################################################
    
    #########################################################
    ### C) problem setup
    #########################################################
    # set up the problem: generate synthetic truth and measurements for the demo 
    # this is not required to call the filters for user-defined models
    setup <- problemsetup(input)
    #########################################################
    
    #########################################################
    ### D) Filters and Smoothers
    #########################################################
    # now apply the filters for state estimation
    # variable to store filter outputs for the comparison plot
    filterout             <- list()
    filterout$runKF       <- input$runKF
    filterout$runEKF      <- input$runEKF
    filterout$runEnKF     <- input$runEnKF
    filterout$runPF       <- input$runPF
    
    #define labels for plotting
    labels      <- c('Elevation', 'Velocity', 'Beta')
    filternames <- c('Kalman Filter' , 'Extended Kalman Filter', 'Ensemble Kalman Filter', 'Particle Filter')
    
    if (input$nonlineardyn == 0) {
      model_handle <- f_model
    } else {
      model_handle <- f_modelnl
    }
    
    if (input$nonlinearmeas == 0) {
      measurement_handle <- h_meas
    } else {
      measurement_handle <- h_measnl
    }
    
    # Kalman Filter
    if (input$runKF==1) {
      print("Exectuing the Kalman filter ")
      Sys.sleep(1)
      
      kalmanout <- kalmanfilter(model_handle, measurement_handle, setup$z, input)
      #plot the output
      plot.filter(kalmanout, truth = setup$xtrue, filtername = filternames[1], label = labels )
      #store for comparison plot
      filterout$kalmanout <- kalmanout  
    } else { filterout$kalmanout <- list() }
    
    # Extended Kalman Filter
    if (input$runEKF==1) {
      print("Exectuing the Extended Kalman filter ")
      Sys.sleep(1)
      
      extendedkalmanout <- extendedkalmanfilter(model_handle, measurement_handle, setup$z, input)
      #plot the output  
      plot.filter(extendedkalmanout, truth = setup$xtrue, filtername = filternames[2], label = labels )
      #store for comparison plot
      filterout$extendedkalmanout <- extendedkalmanout
    } else {filterout$extendedkalmanout <- list() }
    
    # Ensemble Kalman Filter
    if (input$runEnKF==1) {
      print("Executing the Ensemble Kalman filter ")
      Sys.sleep(1)
      
      ensemblekalmanout <- ensemblekalmanfilter(model_handle, measurement_handle, setup$z, input)
      #plot the output
      plot.filter(ensemblekalmanout, truth = setup$xtrue, filtername = filternames[3], label = labels )
      #store for comparison plot
      filterout$ensemblekalmanout <- ensemblekalmanout
    } else { filterout$ensemblekalmanout <- list() }
    
    # Partile filter
    if (input$runPF==1) {
      print("Exectuing the Particle filter ")
      Sys.sleep(1)
      
      particleout <- particlefilter(model_handle, measurement_handle, setup$z, input) 
      #plot the output
      plot.filter(particleout, truth = setup$xtrue, filtername = filternames[4], label = labels )
      #store for comparison plot
      filterout$particleout <- particleout  
    } else { filterout$particleout <- list() }
    #########################################################
    
    #########################################################
    ### E) Plotting results
    #########################################################
    if (input$plottype == 1) {
      print("Plotting the comparison plot ")
      Sys.sleep(1)
      comparisonplot(filterout, setup$xtrue, filternames, labels)
    }
  }
