#' @name filterperformance
#' @aliases filterperformance 
#' @title Assess the performance of various filters for any test problem
#' @description This function assesses the ability of a filter to estimate the state variables and/or unknown parameters of a filter. Two criteria are assessed: 1) time taken to identify the state variables within ten percent of their values and 2) ability to track a key state variable of interest
#' @param object  An output of filter class
#' @param truth The true values of the variables (in case of a synthetic experiment or the demostration example)
#' @param varindex The index of the key variable to be used in assessing performance using criteria 2
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  A list with values of both criteria
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso \code{\link{kalmanfitler}} \code{\link{extendedkalmanfitler}} \code{\link{ensemblekalmanfitler}} \code{\link{particlefitler}}
#' @examples  
#' model_handle       <- f_model
#' measurement_handle <- h_meas
#' input              <- setdefault()
#' setup              <- problemsetup(input)
#' kalmanout          <- dafilter(model_handle, measurement_handle, setup, input)
#' kalperf            <- filterperformance(kalmanout, setup$xtrue, mcmcout)
#' exkalperf          <- filterperformance(exkalmanout, setup$xtrue, mcmcout)
#' enkalperf          <- filterperformance(enkalmanout, setup$xtrue, mcmcout)
#' partperf           <- filterperformance(partout, setup$xtrue, mcmcout)
#' precalibperf       <- filterperformance(precalib, setup$xtrue, mcmcout)
#' @export

filterperformance <-
  function(object,truth,mcmcout=NULL,varindex=NULL)
  { # estimate the size
    N              <- dim(object$x_mean)[2]
    if (is.null(object$x_for_mean)==TRUE) {
      rmse1         <- NA 
      rmse2         <- NA
    } else {
      numvars        <- dim(object$x_for_mean)[1]
      # estimate the number of variables that were observed
      numobs         <- dim(object$x_for_mean)[1]
      rmsebegin      <- 1
      # criteria 1. Root mean squared error on each filter variable
      rmse1          <- rep(NA, numvars)
      for (i in 1:numvars) {
        rmse1[i] = sqrt(sum((object$x_for_mean[i,rmsebegin:N] - truth[i,rmsebegin:N])^2)/(N))      
      }
      # criteria 2. Root mean squared error between predictions from variables estiamted from MCMC's last 50% members and the filter variables
      rmse2         <- rep(NA, numvars)
      if (is.null(mcmcout)==FALSE) {
        for (i in 1:numvars) {
          rmse2[i] = sqrt(sum((object$x_for_mean[i,rmsebegin:N] - mcmcout$x_for_mean[i,rmsebegin:N])^2)/(N))      
        }
      }
    }
    rmse = list(rmse1=rmse1 ,rmse2=rmse2)
    return(rmse)
    # cleanup
    rm(sz, N, numvars, numobs, rmse)
  }