#' @name logposteriorlake
#' @aliases logposteriorlake
#' @title The log of the posterior for the lake model
#' @description This function estimates the full posterior for the lake model, to be used for MCMC analysis
#' @param beta The list of parameters that MCMC needs to identify
#' @param input Any additional inputs for the lake model, same as the output from setlakedefault()
#' @param yobs The vector of observations
#' @details %%  ~~ If necessary, more details than the dcription above ~~
#' @return  log of the posetrior (unnormalized)
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @seealso \code{\link{basecasesetup}} \code{\link{setbasecasedefault}}
#' @examples %%  ~~any examples~~
#' @importFrom mvtnorm dmvnorm

logposteriorlake <-
  function(beta,input,yobs,xstate=NULL,begintime=1,endtime,option=0,priornext=NULL) {
    # case 1.  the metropolis algorithm uses all observatiosn at once, it learns about initial state, and parameter together
    # in this case, beta includes both states and parameters
    # case 2. the metropolis algorithm only learns one observation at a time, except the first time step, it only learns about the parameters 
    # the state values are dependent upon the previous state, stored in xstate, which is set to NULL for case 1 and time=1 in case 2    
    model_handle  <- input$model_handle
    R             <- input$R
    prior         <- input$prior
    timestep      <- input$timestep
    Qmod          <- input$Qmod
    numstates     <- input$numstates
    numpars       <- input$numpars
    qtype         <- input$qtype
    seed          <- input$seed
    Bmat          <- input$Bmat
    uinput        <- input$uinput
    numbeg        <- numstates+1
    numvars       <- numpars+numstates
    Qstates       <- getQ(Qmod[1:numstates], numstates)
    Qpars         <- getQ(Qmod[numbeg:numvars], numpars)
    if (is.null(xstate)==TRUE) {
      xguess        <- c(input$xguess,input$xguesspars)
      Pxguess       <- c((input$Pxguess),diag(input$Pxguesspars))      
    } else  {
      xguess        <- input$xguesspars
      Pxguess       <- input$Pxguesspars      
    }
    if (is.null(priornext)==TRUE) { # estimate the priors
      priornext   <- list(meanx=xguess,varx=Pxguess)
    }
    logprior      <- logpriorlake(beta,priornext,prior)    
    # setting up variables related to the synthetic truth
    timerange     <- seq(begintime,endtime,by=timestep)  # define the timerange to run the model
    N             <- length(timerange)                   # total number of time steps    
    # Calculate state vector over time from dynamics.
    xpred         <- array(NA,dim=c(numvars,N))
    if (is.null(xstate)==TRUE) {
      xpred[,1]               <- beta    
    } else {
      xpred[1:numstates,1]    <- xstate      
      xpred[numbeg:numvars,1] <- beta
    }    
    # need to run the model only if endtime is greater than begintime, else just pass on the initial setting for likelihood calculation
    if (endtime>=begintime) {
      # running base model
      for (i in begintime:endtime) {
        tempi     <- i - begintime + 1
        if (tempi==1) {
          xpred[1:numstates,tempi]   <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+tempi,xpred[1:numstates,tempi],xpred[numbeg:numvars,tempi],timestep,tempi)+Bmat*uinput[i]      
          xpred[numbeg:numvars,tempi]<- xpred[numbeg:numvars,tempi]
        } else {
          xpred[,tempi]              <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+tempi,xpred[1:numstates,tempi-1],xpred[numbeg:numvars,tempi-1],timestep,tempi)+Bmat*uinput[i]      
          xpred[numbeg:numvars,tempi]<- xpred[numbeg:numvars,tempi-1]
        }
      }
    }        
    # now estimate the likelihood
    # The observation error is assumed to be normal with variance R. 
    # The likelihood of an observation given the measurement z[time] is simply
    # the probability distribution function of the normal distribution with variance R
    ypred    <- xpred[1,]
    loglike  <- sum(dnorm(yobs, ypred, sqrt(R), log =TRUE))    
    # make the likelihood a large negative number if any NaNs are produced
#     print(loglike)
    if(length(which(is.nan(ypred) == TRUE)) > 1 || length(which(is.na(ypred) == TRUE)) ) {
      loglike <- -Inf
    } 
    # make the likelihood a large negative number if ranges of b and q parameters are unrealistic
    if (length(beta)==3) {
      if (beta[1]<0.0 || beta[1]>0.5) {
        loglike <- -Inf 
      }
      if (beta[2]<0.0 || beta[2]>1.0) {
        loglike <- -Inf 
      }
      if (beta[3]<2 || beta[3]>8) {
        loglike <- -Inf 
      }
    } else {
      if (beta[1]<0.0 || beta[1]>1.0) {
        loglike <- -Inf 
      }
      if (beta[2]<2 || beta[2]>8) {
        loglike <- -Inf 
      }
    }
  if (option == 0) {
      return(logprior + loglike)    
    } else if (option==1) {
      return(loglike)
    } else if (option==2) {
      return(logprior)
    }    
  }