#' @name paperplot2_Sup
#' @aliases paperplot2_Sup
#' @title Runs the code for the generating Figure 2(supplementary version) in the publication
#' @description This function runs the code for generating Figure 2's supplement with different input policies.  The figure shows the trajectories of state variables with time for each method.
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @export

paperplot2_Sup <- 
  function(option=2,filterout=NULL) {
    if (is.null(filterout)==TRUE) {
      # assign function handles
      input          <- setlakedefault(c(1,1),option)
      logposteriormod<- input$logposteriormod      
      #estimate the sizes and assign arrays
      nummethods     <- 4                                # number of methods to run is 6
      numvars        <- input$numpars + input$numstates  # size of the state variable vector: phosphorus, b, and q, parameters 
      N              <- length(seq(1,input$tend))        # total number of time steps to simulate      
      # initiate filter with choice of seeds for generating the measurement noise and process noise (if not zero)
      input          <- setlakedefault(c(1,1),option)
      setup          <- lakesetup(input)
      # run the filters
      kalout         <- kalmanfilter(input,setup$z)
      exkalout       <- extendedkalmanfilter(input,setup$z)
      enkalout       <- ensemblekalmanfilter(input,setup$z)
      partout        <- particlefilter(input,setup$z)
      # run precalibration
      precalout      <- precalibration(input,setup,input$tend,input$tend) 
      # run the mcmc      
      mcmcout        <- runmcmc(logposteriormod,input,setup,input$tend,input$tend,0)     
      # save output in a single list for plotting later
      filterout                   <- list()
      filterout$runKF             <- input$runKF
      filterout$runEKF            <- input$runEKF
      filterout$runEnKF           <- input$runEnKF
      filterout$runPF             <- input$runPF
      filterout$kalmanout         <- kalout  
      filterout$extendedkalmanout <- exkalout
      filterout$ensemblekalmanout <- enkalout
      filterout$particleout       <- partout 
      filterout$precalib          <- precalout
      filterout$mcmcout           <- mcmcout
      rm(kalout,exkalout,enkalout,partout,precalout,mcmcout)      
      # save the data
      if (option==2) {
        # use 1a for no-flipping case and 1b for the flipping case
        save(filterout, file = 'paperplot2c.Rdata')
      } else if (option==3) {
        save(filterout, file = 'paperplot2d.Rdata')
      } else if (option==4) {
        save(filterout, file = 'paperplot2e.Rdata')
      } else if (option==5) {
        save(filterout, file = 'paperplot2f.Rdata')
      }
    } else {
      input          <- setlakedefault(c(1,1),option)
      setup          <- lakesetup(input)
      #estimate the sizes and assign arrays
      nummethods     <- 4                                # number of methods to run is 6
      numvars        <- input$numstates+input$numpars    # size of the state variable vector: phosphorus, b, and q, parameters      
    }    
    # now plot, use 1a for the no-flipping case, and 1b for the flipping case
    if (option==2) {
      cairo_pdf(paste("Figure2c.pdf", sep=""), width=6, height=6)
    } else if (option==3) {
      cairo_pdf(paste("Figure2d.pdf", sep=""), width=6, height=6)
    } else if (option==4) {
      cairo_pdf(paste("Figure2e.pdf", sep=""), width=6, height=6)
    } else if (option==4) {
      cairo_pdf(paste("Figure2f.pdf", sep=""), width=6, height=6)
    }
      
    # define labels for plotting
    object      <- filterout
    labels      <- input$labels
    filternames <- c('Kalman Filter','Extended Kalman Filter','Ensemble Kalman Filter','Particle Filter',
                     'Precalibration','MCMC')
    # default filter colors
    fcolors     <-  c(rgb(1,0.5,0),rgb(1,0,0),rgb(0,0.8,0),rgb(0,1,1),rgb(0.5,0.5,0.5),rgb(0,0,1))
    lwdval      <-  c(2,2,2,2,2,2)
    ltyval      <-  c(1,1,1,1,1,1)
    # define variables
    truth        <- setup$xtrue
    observations <- setup$z
    numobs       <- 1
    nummethods   <- 4
    #range of x-axis
    xrange       <-  seq(0,input$tend, input$timestep)
    N            <-  length(xrange)
    # initialize plot
    # start plot
    par(family = "sans")
    par(cex = 1.5)
    par(mar = c(3.5,3.5,1.5,0.5)+0.1)
    par(mgp = c(2.2,1.2,0))
    par(bg = NA)
    # create the plot
    mat <- matrix(seq(1,numvars,length = numvars),numvars,1,byrow = TRUE)
    layout(mat,widths = c(rep(1,numvars)),heights = c(rep(1, numvars)))
    for (plt in 1:numvars) {
      # estimate the range of y axis based on the range of analysis mean and observations (if applicable)
      if (numobs>= plt) {
        if (numobs == 1) {
          yrange <- range(c(object$kalmanout$x_mean[plt , ],object$extendedkalmanout$x_mean[plt , ],
                            object$ensemblekalmanout$x_mean[plt , ], object$particleout$x_mean[plt , ], observations), na.rm=TRUE)
        } else {
          yrange <- range(c(object$kalmanout$x_mean[plt , ],object$extendedkalmanout$x_mean[plt , ],
                            object$ensemblekalmanout$x_mean[plt , ], object$particleout$x_mean[plt , ], observations[plt,]), na.rm=TRUE)          
        }
      } else {
        yrange   <- range(object$kalmanout$x_mean[plt , ],object$extendedkalmanout$x_mean[plt , ],
                          object$ensemblekalmanout$x_mean[plt , ], object$particleout$x_mean[plt , ], na.rm=TRUE)
      }     
      # if the truth is provided, estimated ranges based on the true values
      if (is.null(truth)==FALSE) {
        yrange   <- range(c(yrange, truth[plt,]), na.rm=TRUE)
      }
      # set yranges for both a and b panels as same
      if (plt==1) {
        yrange  <- c(0, 3.0)
      } else if (plt==2) {
        yrange  <- c(0.0, 1.0)
      } else {
        yrange  <- c(2.0, 8.0)
      }
      # create the plot
      if (plt==3) {
        plot(xrange, rep(0,length=length(xrange)),col="white",lwd=1,lty=1,type="l",ylim=yrange,
           ylab=paste(labels[plt], " [units]"),xlab="Time [years]",main=" ",
           cex.lab=1.5, frame.plot=FALSE)
      } else {
        plot(xrange, rep(0,length=length(xrange)),col="white",lwd=1,lty=1,type="l",ylim=yrange,
             ylab=paste(labels[plt], " [units]"),xlab="",main=" ",
             cex.lab=1.5, frame.plot=FALSE)      }
      # now plot the filters' updated means
      if (plt == 1) {
        lines(xrange,c(input$xguess,object$precalib$x_mean[plt,]),         col=fcolors[5],lwd=lwdval[5],lty=ltyval[5]) 
        lines(xrange,c(input$xguess,object$ensemblekalmanout$x_mean[plt,]),col=fcolors[3],lwd=lwdval[3],lty=ltyval[3]) 
        lines(xrange,c(input$xguess,object$particleout$x_mean[plt,]),      col=fcolors[4],lwd=lwdval[4],lty=ltyval[4]) 
        lines(xrange,c(input$xguess,object$mcmcout$x_mean[plt,]),          col=fcolors[6],lwd=lwdval[6],lty=ltyval[6])        
        # plot the truth 
        lines(xrange,c(input$xtruestate,truth[plt,]),col=rgb(0,0,0),lwd=3,lty=3)
      } else {
        parcnt <- plt-1
        lines(xrange,c(input$xguesspars[parcnt],object$precalib$x_mean[plt,]),         col=fcolors[5],lwd=lwdval[5],lty=ltyval[5]) 
        lines(xrange,c(input$xguesspars[parcnt],object$ensemblekalmanout$x_mean[plt,]),col=fcolors[3],lwd=lwdval[3],lty=ltyval[3]) 
        lines(xrange,c(input$xguesspars[parcnt],object$particleout$x_mean[plt,]),      col=fcolors[4],lwd=lwdval[4],lty=ltyval[4]) 
        lines(xrange,c(input$xguesspars[parcnt],object$mcmcout$x_mean[plt,]),         col=fcolors[6],lwd=lwdval[6],lty=ltyval[6])              
        # plot the truth 
        lines(xrange,c(input$xtruepars[parcnt],truth[plt,]),col=rgb(0,0,0),lwd=3,lty=3)        
      }      
      # plot the observations
      if (numobs>=plt) {
        if (numobs == 1) {
          lines(xrange[2:N],observations,col="black", type = "p")
        } else {
          lines(xrange[2:N],observations[plt,],col="black", type = "p")
        }
      }      
      # plot the legend
      if (plt==2 && option==2) {                 
        legend(max(xrange)*2/7,max(yrange),c(filternames[3:6],"Observations", "Truth"),
               horiz=FALSE, box.col='black',y.intersp=1,x.intersp = 1.0, lwd=c(lwdval[3:6],0,3),
               lty=c(ltyval[3:6],0,3), pch=c(rep('', nummethods),'O',' '), col=c(fcolors[3:6],"black",rgb(0,0,0)),
               cex=1.2,ncol=2)   
      }
      if (plt==1 && option==2) {
        mtext('High emissions (linear)', side=3, line=-0.3, adj=0.08)
        mtext('(a)', side=3, line=-0.3, adj=0)
      } else if (plt==2 && option==2) {
        mtext('(b)', side=3, line=0.1, adj=0)      
      } else if (plt==3 && option==2) {
        mtext('(c)',side=3, line=0.1, adj=0)
      } else if (plt==1 && option==3) {
        mtext('High emissions (step1)', side=3, line=-0.3, adj=0.08)
        mtext('(d)', side=3, line=-0.3, adj=0)    
      } else if (plt==2 && option==3) {
        mtext('(e)', side=3, line=0.1, adj=0)      
      } else if (plt==3 && option==3) {
        mtext('(f)', side=3, line=0.1, adj=0)
      }  else if (plt==1 && option==4) {
        mtext('High emissions (step2)', side=3, line=-0.3, adj=0.08)
        mtext('(d)', side=3, line=-0.3, adj=0)    
      } else if (plt==2 && option==4) {
        mtext('(e)', side=3, line=0.1, adj=0)      
      } else if (plt==3 && option==4) {
        mtext('(f)', side=3, line=0.1, adj=0)
      }
    }             
    dev.off()
  }