#' @name paperplot4par_Sup
#' @aliases paperplot4par_Sup
#' @title This function runs the code for generating Figure 4's supplementary versions.
#' @description Runs the code for the convergence plot for the ensemble Kalman filter, particle filter, pre-calibration, and MCMC.
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @export

paperplot4par_Sup <- 
  function(option=2) {
    # initiate filter for the lake model, note random seeds are fixed so that the same measurements are used for performance assessment
    input          <- setlakedefault(c(1,1),option)
    setup          <- lakesetup(input)
    # for the linear model
    f_model        <- input$model_handle
    h_model        <- input$measurement_handle
    logposteriormod<- input$logposteriormod
    mcmcmult       <- (input$tend*(input$tend+1))/(2*input$tend)
    # define the number of ensembles/ particles / samples
    if (option==0) {
      numsample    <- c(10,25,50,75,100,250,500,750,1000,2500,5000,10000,15000,20000)         
    } else {
      numsample    <- c(25,50,75,100,250,500,750,1000,2500,5000,10000,15000,20000)         
    }
    sampleend      <- length(numsample)
    numtimes       <- 10    
    # identify the number of samples for MCMC
    newnumsample   <- ceiling(numsample*mcmcmult)
    # identify the extra samples to be run for EnKF, PF, and PreCal
    startind       <- min(which((newnumsample>numsample[sampleend])==TRUE))+1
    # after running the code, R threw memory errors for samples > 252500, so samples are restricted to this number
    # sampleend      <- max(sampleend-1,startind)
    extdnumsample  <- c(numsample,newnumsample[startind:sampleend])
    # define the number of times to repeat an experiment
    numsampleln    <- length(numsample)
    numsamplenext  <- length(extdnumsample)
    numtimesMCMC   <- length(numsample)*numtimes
    numtimesREM    <- length(extdnumsample)*numtimes
    # estimate the number of varaibles
    numvars        <- input$numpars+input$numstates
    # initiate some variables
    meansetup      <- rep(NA,numtimes)    
    # run parallel: ensemble Kalman
    no_cores       <- detectCores() - 1
    cl             <- makeCluster(numtimes,outfile="debug.txt")
    registerDoParallel(cl)
    # for case B, enkal begins at period=10 as some synthetic realizations dont work with smaller samples
    # index from which to start samples as EnKF tanks for smaller number of samples
    beginind       <- 1
    if (option>0) {periodstart <- beginind
    } else {
      periodstart <- 1 
    }
    enkalout   <- foreach(itr=1:numtimes,.packages='learningmodule',.combine='rbind') %:% foreach(period=periodstart:numsamplenext) %dopar% {
      # initiate filter      
      input            <- setlakedefault(c(itr,itr),option)
      input$n_ens      <- extdnumsample[period]
      setup            <- lakesetup(input)
      # run the filter
      enkout           <- ensemblekalmanfilter(input,setup$z)
      enkperfall       <- filterperformance(enkout,setup$xtrue)
      enktime          <- as.numeric(enkout$timetaken)
      # get the performance values
      enkperf          <- enkperfall$rmse1[1]
      return(list(enktime=enktime,enkperf=enkperf))
    }      
    # end parallelization 
    stopCluster(cl)        
    # run parallel: particle
    no_cores       <- detectCores() - 1
    cl             <- makeCluster(numtimes,outfile="debug2.txt")
    registerDoParallel(cl)
    # particle filter
    partout   <- foreach(itr=1:numtimes,.packages='learningmodule',.combine='rbind') %:% foreach(period=1:numsamplenext) %dopar% {
      # initiate filter
      input      <- setlakedefault(c(itr,itr),option)
      input$n_par<- round(extdnumsample[period])
      setup      <- lakesetup(input)
      # run the filter
      prtout         <- particlefilter(input,setup$z)
      prtperfall     <- filterperformance(prtout,setup$xtrue)
      prttime        <- as.numeric(prtout$timetaken)
      # get the performance values
      prtperf        <- prtperfall$rmse1[1]
      list(prttime=prttime,prtperf=prtperf)
    } 
    # end parallelization 
    stopCluster(cl)            
    # run parallel: precalibrations
    no_cores       <- detectCores() - 1
    cl             <- makeCluster(numtimes,outfile="debug3.txt")
    registerDoParallel(cl)
    # precalibration    
    precalout   <- foreach(itr=1:numtimes,.packages='learningmodule',.combine='rbind') %:% foreach(period=1:numsamplenext) %dopar% {
      # initiate filter      
      input            <- setlakedefault(c(itr,itr),option)
      input$n_precalib <- extdnumsample[period]
      setup            <- lakesetup(input)
      # run the filter
      precout        <- precalibration(input,setup,input$tend,input$tend) 
      precperfall    <- filterperformance(precout,setup$xtrue)
      prectime       <- as.numeric(precout$timetaken)
      # get the performance values
      precperf        <- precperfall$rmse1[1]
      list(prectime=prectime,precperf=precperf)
    }         
    # end parallelization 
    stopCluster(cl)             
    # run parallel: mcmc
    cl             <- makeCluster(numtimes,outfile="debug4.txt")
    registerDoParallel(cl)
    # mcmc
    mcmcout   <- foreach(itr=1:numtimes,.packages='learningmodule',.combine='rbind') %:% foreach(period=1:numsampleln) %dopar% {
      # initiate filter
      input          <- setlakedefault(c(itr,itr),option)
      input$nummcmc  <- numsample[period]
      setup          <- lakesetup(input)
      # run the filter
      mcout          <- runmcmc(logposteriormod,input,setup,input$tend,input$tend,0) 
      mcmcperfall    <- filterperformance(mcout,setup$xtrue)
      mcmctime       <- as.numeric(mcout$timetaken)
      # get the performance values
      mcmcperf       <- mcmcperfall$rmse1[1]
      list(mcmctime=mcmctime,mcmcperf=mcmcperf)
    }          
    # end parallelization 
    stopCluster(cl)        
    for (itr in 1:numtimes) {  
      input            <- setlakedefault(c(itr,itr),option)
      setup            <- lakesetup(input)
      # for normalization
      meansetup[itr]   <- mean(setup$z)
    }
    # assign simpler vectors
    enkalperf    <- array(NA,c(numtimes,numsamplenext))
    enkaltime    <- array(NA,c(numtimes,numsamplenext))
    partperf     <- array(NA,c(numtimes,numsamplenext))
    parttime     <- array(NA,c(numtimes,numsamplenext))
    precalperf   <- array(NA,c(numtimes,numsamplenext))
    precaltime   <- array(NA,c(numtimes,numsamplenext))    
    mcmcperf     <- array(NA,c(numtimes,numsampleln))
    mcmctime     <- array(NA,c(numtimes,numsampleln))
    for (itr in 1:numtimes) {
      for (sample in 1:numsamplenext) {
        if (option>0) {
          if (sample>=beginind) {
            samplenew              <- sample-beginind+1
            enkalperf[itr,sample]  <- enkalout[[itr,samplenew]]$enkperf
            enkaltime[itr,sample]  <- enkalout[[itr,samplenew]]$enktime        
          }          
        } else {
          enkalperf[itr,sample]  <- enkalout[[itr,sample]]$enkperf
          enkaltime[itr,sample]  <- enkalout[[itr,sample]]$enktime                 
        }
        partperf[itr,sample]   <- partout[[itr,sample]]$prtperf
        parttime[itr,sample]   <- partout[[itr,sample]]$prttime       
        precalperf[itr,sample] <- precalout[[itr,sample]]$precperf
        precaltime[itr,sample] <- precalout[[itr,sample]]$prectime
        if (sample<=numsampleln) {
          mcmcperf[itr,sample]   <- mcmcout[[itr,sample]]$mcmcperf
          mcmctime[itr,sample]   <- mcmcout[[itr,sample]]$mcmctime
        }
      }
    }
    # save the data
    finalallperf <- list(enkalperf=enkalperf,partperf=partperf,precalperf=precalperf,mcmcperf=mcmcperf, 
                         enkaltime=enkaltime,parttime=parttime,precaltime=precaltime,mcmctime=mcmctime,
                         extdnumsample=extdnumsample,numtimes=numtimes,meansetup=meansetup)
    if (option==2) {    
      save(finalallperf, file = 'paperplot4c.RData') # c: eutrophic-linear, d: step increase before eutrophication, e: step increase after eutrophication
      # save all files, just a backup
      save(list=ls(all.names=TRUE),file="paperplot4cWRKSPC.RData") # save a snapshop of the workspace just for reference
      ###############################################################################    
      # now plot
      cairo_pdf(paste("Figure4c.pdf", sep=""), width=8, height=4)
      lnnew        <- numsampleln
      newnumsample <- newnumsample[1:lnnew]
    } else if (option==3) {
      save(finalallperf, file = 'paperplot4d.RData') # c: eutrophic-linear, d: step increase before eutrophication, e: step increase after eutrophication
      save(list=ls(all.names=TRUE),file="paperplot4dWRKSPC.RData") # save a snapshop of the workspace just for reference
      # now plot
      cairo_pdf(paste("Figure4d.pdf", sep=""), width=8, height=4)
      lnnew        <- numsampleln
      newnumsample <- newnumsample[1:lnnew]      
    } else if (option==4) {
      save(finalallperf, file = 'paperplot4e.RData') # c: eutrophic-linear, d: step increase before eutrophication, e: step increase after eutrophication
      save(list=ls(all.names=TRUE),file="paperplot4eWRKSPC.RData") # save a snapshop of the workspace just for reference
      # now plot
      cairo_pdf(paste("Figure4e.pdf", sep=""), width=8, height=4)
      lnnew        <- numsampleln
      newnumsample <- newnumsample[1:lnnew]      
    }
    enkalperf  <- 100*finalallperf$enkalperf/mean(finalallperf$meansetup)
    partperf   <- 100*finalallperf$partperf/mean(finalallperf$meansetup)
    precalperf <- 100*finalallperf$precalperf/mean(finalallperf$meansetup)
    mcmcperf   <- 100*finalallperf$mcmcperf/mean(finalallperf$meansetup)
    perf       <- c(range(enkalperf,na.rm=TRUE),range(partperf,na.rm=TRUE),range(precalperf,na.rm=TRUE),range(mcmcperf,na.rm=TRUE))
    ylimrange  <- round(range(c(enkalperf,partperf,precalperf,mcmcperf),na.rm=TRUE))    
    jitter     <- c(-1,-0.5,0.5,1)
    # labels and colors
    label      <- c('EnsembleK','Particle','Precalib','MCMC')    
    cols       <- c(rgb(0,0.8,0),rgb(0,1,1),rgb(0.4,0.4,0.4),rgb(0,0,1))
    colslines  <- c(rgb(0,0.8,0,0.5),rgb(0,1,1,0.5),rgb(0.4,0.4,0.4,0.5),rgb(0,0,1,0.5))
    pchvals    <- c(1,1,1,1)
    # define figure margins:
    par(mar=c(4,4,1,1))
    par(mgp=c(2.5,1,0))
    plot(extdnumsample,rep(mean(perf,na.rm=TRUE),numsamplenext),log='xy', 
         ylab = "RMSE [%]",xlab = "Ensembles [x100 function evaluations]",cex.lab=1.5,
         axes=FALSE,cex=2, pch= 21, col=rgb(1,1,1), bg=rgb(1,1,1),
         ylim=ylimrange,xlim=range(c(numsample,mcmcmult*numsample)))    
    # plotting mean performance across seeds
    points(extdnumsample,apply(enkalperf,2,mean), pch=pchvals[1],bg=cols[1],col=cols[1],cex=2)
    points(extdnumsample,apply(partperf,2,mean),  pch=pchvals[2],bg=cols[2],col=cols[2],cex=2)
    points(extdnumsample,apply(precalperf,2,mean),pch=pchvals[3],bg=cols[3],col=cols[3],cex=2)
    points(newnumsample,apply(mcmcperf,2,mean)[1:lnnew],   pch=pchvals[4],bg=cols[4],col=cols[4],cex=2)
    # connecting the points by lines
    lines(extdnumsample,apply(enkalperf,2,mean), col=cols[1],lwd=2)
    lines(extdnumsample,apply(partperf,2,mean),  col=cols[2],lwd=2)
    lines(extdnumsample,apply(precalperf,2,mean),col=cols[3],lwd=2)
    lines(newnumsample,apply(mcmcperf,2,mean)[1:lnnew],   col=cols[4],lwd=2)    
    # now plotting distribution of performance across seeds
    for (sample in 1:numsamplenext) {
      lines(c(extdnumsample[sample]+jitter[1],extdnumsample[sample]+jitter[1]),range(enkalperf[,sample]),col=colslines[1],lwd=2)
      lines(c(extdnumsample[sample]+jitter[2],extdnumsample[sample]+jitter[2]),range(partperf[,sample]),col=colslines[2],lwd=2)
      lines(c(extdnumsample[sample]+jitter[3],extdnumsample[sample]+jitter[3]),range(precalperf[,sample]),col=colslines[3],lwd=2)
      if (sample<=numsampleln) {
        lines(c(newnumsample[sample]+jitter[4],newnumsample[sample]+jitter[4]),range(mcmcperf[,sample]),col=colslines[4],lwd=2)
      }
    }
    # axis labels
    axis(side=1,at=extdnumsample,labels=extdnumsample)
    axis(side=2,at=c(1,5,25,50,100),labels=c(1,5,25,50,100))
    # legend only for the eutrophication plot
    if (option==2) {
      mtext('High emissions (linear)', side=3, adj=0.0, line=-0.4, cex=1.3)
    } else if (option==3) {
      mtext('High emissions (step1)', side=3, adj=0.0, line=-0.4, cex=1.3)  
    } else if (option==4) {
      mtext('High emissions (step2)', side=3, adj=0.0, line=-0.4, cex=1.3)
    }
    legend(18,8,c("EnsembleK","Particle","Precalib","MCMC"),
           horiz=FALSE,box.col= rgb(0,0,0,1),bg=rgb(0,0,0,0),y.intersp=0.9,x.intersp=0.4,lty=c(0,0,0,0),
           pch=pchvals,col=cols,pt.bg=cols,pt.cex=c(2,2,2,2),lwd=c(2,2,2,2),cex=1.2,ncol=2)
    # add a horizontal straight line beginning from MCMC's best performer going backward for visual comparison
 #   lines(c(extdnumsample[1],extdnumsample[numtimesREM]), c(mcmcperf[numtimesMCMC],mcmcperf[numtimesMCMC]),col=rgb(0.5,0.5,0.5),lty=3,lwd=2)
    dev.off()
  }