#' @name particlefilter
#' @aliases particlefilter
#' @title Implement the particle filter
#' @description This function implements the particle filter for a given set of model inputs and observations. The filter approximates a Gaussian distribution using a set of randomly generated, and is a good choice for models with significant non linearity
#' @param input A list with the problem setup including the first guess of forecast error covariance matrix (Pguess),  first guess values of forecast vector (xguess), model process error covariance matrix (Qmod), measurement covaraince (R), and, time step of model (timestep)
#' @param observations The set of observations that the filter will use to update its estimates.
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  A filter class which is a list comprising of the mean of the forecasted variables and their associated covariance matrix
#' @references %% ~put references to the literature/web site here ~
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso  \code{\link{kalmanfilter}}, \code{\link{extendedkalmanfilter}}, \code{\link{ensemblekalmanfilter}}
#' @examples
#' input              <- setdefault()
#' setup              <- problemsetup(input)
#' particleout        <- particlefilter(input,setup$z)
#' @importFrom numDeriv jacobian
#' @importFrom mvtnorm dmvnorm
#' @export

particlefilter <-
  function(input,observations)
  {
    # starting time counter
    start.time           <- Sys.time()    
    # assign function handles
    model_handle         <- input$model_handle
    measurement_handle   <- input$measurement_handle
    param_handle         <- input$param_handle
    parammeas_handle     <- input$parammeas_handle
    # assign variables
    xguess                <- input$xguess
    Pxguess               <- input$Pxguess
    xguesspars            <- input$xguesspars
    Pxguesspars           <- input$Pxguesspars
    Qmod                  <- input$Qmod
    R                     <- input$R
    timestep              <- input$timestep
    Bmat                  <- input$Bmat
    uinput                <- input$uinput
    numpars               <- input$numpars
    numstates             <- input$numstates
    n_par                 <- input$n_par
    resamplechoice        <- input$resamplechoice
    N                     <- length(seq(input$tbeg, input$tend, timestep))       
    observations          <- as.matrix(observations)
    Pxguess               <- as.matrix(Pxguess)
    xguess                <- as.numeric(xguess)
    Pxguesspars           <- as.matrix(Pxguesspars)
    xguesspars            <- as.numeric(xguesspars)
    n_par                 <- as.numeric(n_par)
    # estimate the process noise matrix
    Qstates               <- getQ(Qmod[1:numstates], numstates)
    numbeg                <- numstates+1
    numvars               <- numpars+numstates
    Qpars                 <- getQ(Qmod[numbeg:numvars], numpars)    
    # intialize variables
    # store all updated ensembles for all variables, and all time steps
    #     x_par_all            <- array(NA, dim=c(numvars,n_par,N))
    #     par_w_all            <- array(NA, dim=c(numvars,n_par,N))
    # store the state and parameter particles members at current time step, set all values at initial guess
    x_par                 <- array(NA, dim=c(numstates,n_par))    
    x_partemp             <- array(NA, dim=c(numstates,n_par))    
    x_parpars             <- array(NA, dim=c(numpars,n_par))    
    # store the state and parameter particle weights at current time step
    par_w                 <- array(1/n_par, dim=c(1,n_par))
    par_wpars             <- array(1/n_par, dim=c(1,n_par))
    # store the deviation of each particle from the mean at current time step
    x_ana_mean            <- array(NA, dim=c(numvars,N))    
    # error covariance matrix after update for all time steps
    ana_cov               <- array(NA, dim=c(numvars,numvars,N))    
    # number of variables that can be measured, and hence forecasted
    numfor                <- length(measurement_handle(x_par[,1]))    
    # store the mean of the distribution, i.e., the mean of the updated ensemble for all time steps
    x_for_mean            <- array(NA, dim=c(numstates,N))
    #error covariance matrix before update for all time steps
    P_forecast            <- array(NA, dim=c(numstates,numstates,N))    
    # tract effective sample size
    neff                  <- array(NA, dim=c(1,N))        
    # store all ensembles across all time periods
    x_parall              <- array(list(), N) 
    # re-assign variables whose size depends on n_par
    x_ana_pert            <- array(NA, dim=c(numvars,n_par))    
    # store the forecasts for all forecasted variables, and ensembles at current time step
    xfor                  <- array(NA, dim=c(numfor,n_par))          
    # generating the state ensemble by sampling a multivariate distribution
    if (input$prior == 0){
      # generate random samples using intial covariance estimate Pguess
      temp                <- multirandnorm(xguess, Pxguess, n_par, input$seed[2])
      # assign
      x_par               <- temp
      rm(temp)
      # if needed: estimate intial weight of all particles based on the multi-variate normal distribution
      # par_w[par]       <- dmvnorm(x_par[,par],mean=xguess,sigma=Pguess)  
    } else {
      location            <- rep(NA, numstates)
      scale               <- rep(NA, numstates)
      for (i in 1:numstates) {
        tempmean          <- xguess[i]
        tempvar           <- Pxguess[i,i]
        location[i]       <- log(tempmean/sqrt(1+tempvar/tempmean^2))
        scale[i]          <- (log(1+tempvar/tempmean^2))
      }
      if(length(scale)==1) {
        scale             <- scale
      } else {
        scale             <- diag(scale)
      }
      # generate random multi-variate noise
      temp                <- multirandnorm(location,scale,n_par,input$seed[2])        
      # estimate the final ensemble
      x_par               <- t(exp(temp))
      rm(temp)
      # if needed: estimate intial weight of all particles based on the multi-variate normal distribution      
      # par_w[par]       <- dmvnorm(log(x_par[,par]),mean=location, sigma=scale)    
    }
    rm(location,scale)    
    # generating the parameter ensemble by sampling a multivariate distribution
    if (input$prior == 0) {
      # generate random samples using intial covariance estimate Pguess
      temp                <- multirandnorm(xguesspars,Pxguesspars,n_par,input$seed[2])
      # assign
      x_parpars           <- temp
      rm(temp)
      # if needed: estimate intial weight of all particles based on the multi-variate normal distribution
      # par_w[par]       <- dmvnorm(x_par[,par],mean=xguess,sigma=Pguess)  
    } else {
      location            <- rep(NA,numpars)
      scale               <- rep(NA,numpars)
      for (i in 1:numpars) {
        tempmean          <- xguesspars[i]
        tempvar           <- Pxguesspars[i,i]
        location[i]       <- log(tempmean/sqrt(1+tempvar/tempmean^2))
        scale[i]          <- (log(1+tempvar/tempmean^2))
      }
      if(length(scale)==1) {
        scale             <- scale
      } else {
        scale             <- diag(scale)
      }
      # generate random particles
      temp                <- multirandnorm(location,scale,n_par,input$seed[2])        
      # estimate the final ensemble
      x_parpars           <- t(exp(temp))
      rm(temp)
      # if needed: estimate intial weight of all particles based on the multi-variate normal distribution      
      # par_w[par]       <- dmvnorm(log(x_par[,par]),mean=location, sigma=scale)    
    }
    rm(location,scale) 
    # remove unrealistic parameter particles
    for (par in 1:numpars) {
      # no parameter can be negative
      if (par==1) {
        temp1               <- which(x_parpars[par,]<0)
      } else {
        temp1               <- c(temp1, which(x_parpars[par,]<0))
      }
      # b parameter cannot be greater than 1
      if (par==1) {
        temp2             <- which(x_parpars[par,]>1)
      } else {
        # q parameter has to be between 2 and 8
        temp3                <- which(x_parpars[par,]<2)
        temp4                <- which(x_parpars[par,]>8)
      }       
    }    
    # remove unrealistic state estimates
    for (i in 1:numstates) {
      # no state can be negative
      if (i==1) {
        temp5             <- which(x_par[i,]<0)
      } else {
        temp5 <- union(temp3, which(x_par[i,]<0))
      }
    }
    temp                      <- union(union(union(union(temp1,temp2),temp3),temp4),temp5)
    if (length(temp)>0) {
      x_parpars               <- x_parpars[,-temp]
      par_wpars               <- par_wpars[-temp]      
      n_par                   <- length(par_wpars) 
      x_par                   <- array(x_par[,-temp],c(numstates,n_par))  
      par_w                   <- par_w[-temp]
      # re-assign variables whose size depends on n_par
      x_ana_pert              <- array(NA, dim=c(numvars,n_par))    
      # store the forecasts for all forecasted variables, and ensembles at current time step
      xfor                    <- array(NA, dim=c(numfor,n_par))          
      # normalize the weights after removing anomalous parameters
      par_wpars               <- par_wpars/sum(par_wpars)
      par_w                   <- par_w/sum(par_w)
      rm(temp,temp1,temp2,temp3,temp4,temp5)
    }
    # Particle filter main loop here
    # Loop through time, and ensembles, iterating over prediction and measurement update
    for (time in 1:N) {
      # Step 1. State prediction
      for (par in 1:n_par) {
        #note: at time = 1, the measurement is made directly from the given model state
        x_partemp[,par]       <- addprocessnoise(model_handle, Qstates, input$qtype, par+n_par*time, x_par[,par], x_parpars[,par], timestep, time) + Bmat*uinput[time]        
      } # store the forecast mean and covariance
      # convert prediction to measurement forecast
      for (par in 1:n_par)
      {      
        xfor[,par]            <- measurement_handle(x_partemp[,par])
      } 
      x_for_mean[,time]       <- xfor%*%array(par_w, dim=c(n_par,1))
      # estimate deviation from the mean for each member of the forecasted particle    
      Pfpert                  <- (xfor  - x_for_mean[,time])^2              
      # P_analysis is the analysis error covariance matrix
      P_forecast[,,time]      <- sum(par_w*Pfpert)
      # Step 2. update weights associated with parameters based on measurement prediction
      # for this, calculate the likelihood associated with each particle for a given measurement at each time step
      # this function updates the particle weights based on the synthetic observations (z)
      lkhd                    <- likelihood_update(xfor,observations[time], R)
      # performing calculations in log space to deal with low weights
      par_wpars               <- exp(log(par_wpars) + lkhd)  
      # normalize the weights
      par_wpars               <- par_wpars/sum(par_wpars)      
      # remove any particles that show a negative parameter value or have b values >1
      for (par in 1:numpars) {
        # no parameter can be negative
        if (par==1) {
          temp1               <- which(x_parpars[par,]<0)
        } else {
          temp1               <- union(temp1, which(x_parpars[par,]<0))  
        }
        # b parameter cannot be greater than 1
        if (par==1) {
          temp2               <- which(x_parpars[par,]>1)          
        } else {
          # q parameter has to be between 2 and 8
          temp3                <- which(x_parpars[par,]<2)
          temp4                <- which(x_parpars[par,]>8)
        } 
      }      
      temp                    <- union(union(union(temp1,temp2),temp3),temp4)
      if (length(temp)>0) {
        x_parpars               <- x_parpars[,-temp]
        par_wpars               <- par_wpars[-temp]      
        n_par                   <- length(par_wpars) 
        x_par                   <- array(x_par[,-temp],c(numstates,n_par))  
        par_w                   <- par_w[-temp]
        # re-assign variables whose size depends on n_par
        x_ana_pert              <- array(NA, dim=c(numvars,n_par))    
        # store the forecasts for all forecasted variables, and ensembles at current time step
        xfor                    <- array(NA, dim=c(numfor,n_par)) 
        # normalize the weights after removing anomalous parameters
        par_wpars               <- par_wpars/sum(par_wpars)
        par_w                   <- par_w/sum(par_w)
        rm(temp,temp1,temp2,temp3,temp4)
      }   
      # estimate effective sample size (if needed for analysis), not being used currently
      neff[time]                <- 1/(sum(par_wpars^2))
      # set threshold for effective particle size - if neff is greater than this, resampling can be skipped otherwise not
      nth                       <- 2*n_par/3
      # resample the particles 
      # resampling is required to prevent from one or a few particles from having too high a probability
      # here we are sampling from the likelihood which is favored in case of high signal to noise ratio      
      if (is.nan(neff[time])) {
        # particle filter has collapsed to a small region, the particles are too few 
        # all simulations from here on are NaNs
        x_parpars                <- array(NaN, dim=c(numpars,n_par))
      } else {
        if (neff[time]< nth) {
          resampled              <- resample(x_parpars,par_wpars, resamplechoice)
          # assign the resampled particles and weights for the next prediction/measurement update step
          x_parpars              <- array(resampled$x, c(numpars, n_par))
          par_wpars              <- resampled$w
          resamplechk            <- TRUE
        }      
        rm(resampled)
      }            
      # Step 4. run the model with resampled particles
      for (par in 1:n_par) {
        #note: at time = 1, the measurement is made directly from the given model state
        x_par[,par]           <- addprocessnoise(model_handle, Qstates, input$qtype, par+n_par*time, x_par[,par], x_parpars[,par], timestep, time) + Bmat*uinput[time]        
      } # store the forecast mean and covariance
      # convert prediction to measurement forecast
      for (par in 1:n_par)
      {      
        xfor[,par]            <- measurement_handle(x_par[,par])
      } 
      # Step 5. Update the state weights based on the predictions from Step 4
      # for this, calculate the likelihood associated with each particle for a given measurement at each time step
      # this function updates the particle weights based on the synthetic observations (z)
      lkhd                    <- likelihood_update(xfor,observations[time], R)
      # performing calculations in log space to deal with low weights
      par_w                   <- exp(log(par_w) + lkhd)      
      # normalize the weights based to form a probability distribution (i.e. sum to 1).
      par_w                   <- par_w/sum(par_w)      
      # no state can be negative
      for (i in 1:numstates) {
        if (i==1) {
          temp                  <- which(x_par[i,]<0)
        } else {
          temp                  <- union(temp,which(x_par[i,]<0))
        }
      }
      if (length(temp)>0) {
        x_parpars               <- x_parpars[,-temp]
        par_wpars               <- par_wpars[-temp]      
        n_par                   <- length(par_wpars) 
        x_par                   <- array(x_par[,-temp],c(numstates,n_par))  
        par_w                   <- par_w[-temp]
        # re-assign variables whose size depends on n_par
        x_ana_pert              <- array(NA, dim=c(numvars,n_par))    
        # store the forecasts for all forecasted variables, and ensembles at current time step
        xfor                    <- array(NA, dim=c(numfor,n_par)) 
        # normalize the weights after removing anomalous parameters
        par_wpars               <- par_wpars/sum(par_wpars)
        par_w                   <- par_w/sum(par_w)
        rm(temp)
      }
      # estimate effective sample size (if needed for analysis), not being used currently
      neff[time]              <- 1/(sum(par_w^2))
      # store variables for plotting, summary statistics and estimate some diagnostic variables
      # calculate signal to noise ratio: maximum value of likelihood / maximum value of prior
      snr                     <- max(par_w)/(1/n_par)   # prior = 1/n_parpars
      # Step 6. resample the state particles before ending calculation at this time step
      # resampling is required to prevent from one or a few particles from having too high a probability
      # here we are sampling from the likelihood which is favored in case of high signal to noise ratio      
      if (is.nan(neff[time])) {
        # particle filter has collapsed to a small region, the particles are too few 
        # all simulations from here on are NaNs
        x_par                    <- array(NaN, dim=c(numstates,n_par))
      } else {
        if (neff[time]< nth) {
          resampled              <- resample(x_par,par_w, resamplechoice)
          # assign the resampled particles and weights for the next prediction/measurement update step
          x_par                  <- array(resampled$x, c(numstates, n_par))
          par_w                  <- resampled$w
          resamplechk            <- TRUE
        }      
        rm(resampled)
      }
      # putting all particles  across time and prediction variables in a single matrix for plotting later on
      #       x_par_all[1:numstates,,time]    <- x_par
      #       x_par_all[numbeg:numvars,,time] <- x_parpars
      #       par_w_all[1:numstates,,time]    <- par_w
      #       par_w_all[numbeg:numvars,,time] <- par_wpars
      # calculate the analysis mean, i.e., the mean of the updated ensemble
      x_ana_mean[1:numstates,time]     <- x_par%*%array(par_w, dim=c(n_par,1))
      x_ana_mean[numbeg:numvars,time]  <- x_parpars%*%array(par_wpars,dim=c(n_par,1))
      # estimate deviation from the analysis mean for each member of the updated ensemble      
      ana_par                   <- rep(NA,numstates)
      ana_parpars               <- rep(NA,numpars)
      for (dims in 1:numstates) {        
        x_ana_pert[dims,]       <- (x_par[dims,]  - x_ana_mean[dims,time])^2        
        ana_par[dims]           <- (sum(par_w*(x_ana_pert[dims,])))        
        ana_cov[dims,dims,time] <- ana_par[dims]
      }
      for (dims in numbeg:numvars) {   
        tempi                   <- dims - numpars + 1
        x_ana_pert[dims,]       <- (x_parpars[tempi,]  - x_ana_mean[dims,time])^2  
        ana_parpars[tempi]      <- (sum(par_wpars*(x_ana_pert[dims,])))   
        ana_cov[dims,dims,time] <- ana_parpars[tempi]
      }
      # ana_cov is the analysis error covariance matrix
#      ana_cov[,,time]           <-  x_ana_pert%*%t(x_ana_pert) 
      x_parall[[time]]          <-  array(c(t(x_par), t(x_parpars)),dim=c(n_par,numvars)) 
    }# end loop over time    
    #return output
    filterest        <- list(name='Particle',x_mean=x_ana_mean,x_cov=ana_cov,x_for_mean=x_for_mean,x_for_cov=P_forecast,x_parall=x_parall)
    filterest$call   <- match.call()
    rm(x_ana_mean,ana_cov,x_for_mean,P_forecast,x_parall)
    #define the fitler class
    class(filterest) <- "filter"
    # ending time counter
    end.time   <- Sys.time()
    filterest$timetaken <- end.time - start.time
    if (input$timeflag == 1) {print(paste("Function time is " , round(as.numeric(filterest$timetaken, units="secs"),2), "seconds"))}    
    filterest
  }