#' @name plot.filter
#' @aliases plot.filter
#' @title Plot function for the filter class
#' @description Plot the variable output from the filter, one sub plot is created for each variable
#' @param object  An output of filter class
#' @param truth The true values of the variables (in case of a synthetic experiment or the demostration example)
#' @param filtername  The name of the filter for the title of the plot
#' @param label The names of the variables being plotted
#' @details %%  ~~ If necessary, more details than the dcription above ~~
#' @return  %% ~~does not return anything ~~
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note      %% add value
#' @seealso   \code{\link{print.filter}}, \code{\link{summary.filter}}
#' @examples
#' #using the demostration example set up
#' model_handle       <- f_model
#' measurement_handle <- h_meas
#' input              <- setdefault()
#' setup              <- problemsetup(input)
#' kalmanout          <- dafilter(model_handle, measurement_handle, setup, input)
#' plot(kalmanout)
#' @export

plot.filter <-
  function(object, truth = NULL, filtername = "Filter ", label = NULL)
  {
    #estimate the size
    sz             <- dim(object$x_mean)
    if (is.null(sz) == TRUE) {
      N            <- length(object$x_mean)
      numvars      <- 1
    } else {
      N            <- max(sz)
      numvars      <- min(sz)
    }
    #estimate the number of variables that were observed
    numobs         <- dim(object$observations)
    if (is.null(numobs) == TRUE) {
      numobs       <- 1
    } else {
      numobs       <- min(numobs)
    }

    #initialize plot
    #start plot
    #x11()
    plot.new()
    par(family="sans")
    par(cex=1.5)
    par(mar=c(4,4.5,2,3)+0.1)
    par(mgp=c(2.5,1,0))
    par(bg=NA)

    #set up sub plots based on number of variables to be plotted
    mat <- matrix(seq(1,numvars,length=numvars),numvars,1,byrow=TRUE)
    layout(mat,widths=rep(1, numvars), heights=rep(1, numvars))

    #set the range on x-axis to the length of analysis mean
    xrange   = seq(1,N)

    for (plt in 1:numvars) {
      #estimate the range of y axis based on the range of analysis mean and observations (if applicable)
      if (numobs>= plt) {
        if (numobs == 1) {
          yrange   = range(c(object$x_mean[plt , ],object$observations), na.rm=TRUE)
        } else {
          yrange   = range(c(object$x_mean[plt , ],object$observations[plt,]), na.rm=TRUE)
        }
      } else {
        yrange   = range(object$x_mean[plt , ], na.rm=TRUE)
      }
      #if the truth is provided, estimate ranges based on the true values
      if (is.null(truth)==FALSE) {
        yrange   = range(c(yrange, truth[plt,]), na.rm=TRUE)
      }

      # create the plot
      if (plt==1) {
        if (is.null(label) == TRUE) {
          plot(xrange, rep(0,length=length(xrange)),col="white",lwd=1,lty=1,type="l",ylim=yrange,
               ylab=paste("Variable", plt, " [units]"),xlab="",main=filtername,cex.lab=1.5)
        } else {
          plot(xrange, rep(0,length=length(xrange)),col="white",lwd=1,lty=1,type="l",ylim=yrange,
               ylab=label[plt],xlab="",main=filtername,cex.lab=1.5)
        }
      } else {
        if (is.null(label) == TRUE) {
          plot(xrange, rep(0,length=length(xrange)),col="white",lwd=1,lty=1,type="l",ylim=yrange,
               ylab=paste("Variable", plt, " [units]"),xlab="",main=" ",cex.lab=1.5)
        } else {
          plot(xrange, rep(0,length=length(xrange)),col="white",lwd=1,lty=1,type="l",ylim=yrange,
               ylab=label[plt],xlab="",main=" ",cex.lab=1.5)
        }
      }

      #plot the truth if provided
      if (is.null(truth) == FALSE) {
        lines(xrange,truth[plt,],col="gray",lwd=2,lty=1)

        if (plt == 1) {
          #insert legend
          legend(max(xrange)*1/20,(3*min(yrange)+max(yrange))/4,c("Filter","1 S.D.","Observations", "Truth"),
                 horiz=TRUE, box.col="white",y.intersp=1,x.intersp = 0.7, lwd=c(1,1,0,1), lty=c(1,4,0,1),
                 pch=c(' ',' ','O',' '), col=c("red","red","black","gray"),cex=1.2)
        }
      } else {
        if (plt == 1) {
          #insert legend
          legend(max(xrange)*1/20,(3*min(yrange)+max(yrange))/4,c("Filter","1 S.D.","Observations"),
                 horiz=TRUE, box.col="white",y.intersp=1,lwd=c(1,1,0),lty=c(1,4,0),
                 pch=c(' ',' ','O'), col=c("red","red","black"),cex=1.2)
        }
      }

      #plot the observations
      if (numobs>=plt) {
        if (numobs == 1) {
          lines(xrange,object$observations,col="black", type = "p")
        } else {
          lines(xrange,object$observations[plt,],col="black", type = "p")
        }
      }

      #estimating summary stats of the filter
      filtersum  = summary(object)
      s          = filtersum$std

      #now plot the filter's updated means
      lines(xrange,(object$x_mean[plt,]),col="red",lwd=2,lty=1)
      lines(xrange,(object$x_mean[plt,]+s[,plt]),col="red",lwd=1,lty=4)
      lines(xrange,(object$x_mean[plt,]-s[,plt]),col="red",lwd=1,lty=4)
    }
  }
