#' @name precalibration
#' @aliases precalibration
#' @title Perform precalibration on selected model
#' @description This function performs precalibration on a selected model. First, model parameters are sampled using assumed priors. Then output from each instance is assessed. If it is within 2 s.d. of the observations, the parameters are accepted, else rejected.
#' @param input The problem's input including the values for measurement nosie, process noise, etc.
#' @param setup The problem's setup, including synthetic observations
#' @param preendtime The time until which to predict 
#' @return  Mean prediction along with shortliste parameter sets 
#' @author Riddhi Singh
#' @note %% add value
#' @seealso \code{\link{basecasesetup}} \code{\link{setbasecasedefault}}
#' @examples %%  ~~any examples~~
#' @export

precalibration <-
  function(input,setup,obsendtime,predendtime=input$tend,x_pars=NULL) {
    # starting time counter
    start.time           <- Sys.time()
    # assign variables
    measurement_handle   <- input$measurement_handle
    model_handle         <- input$model_handle  
    Qmod                 <- input$Qmod
    R                    <- input$R
    timestep             <- input$timestep
    Bmat                 <- input$Bmat
    uinput               <- input$uinput
    numpars              <- input$numpars
    numstates            <- input$numstates
    Pxguess              <- input$Pxguess
    xguess               <- input$xguess
    Pxguesspars          <- input$Pxguesspars
    xguesspars           <- input$xguesspars
    n_precalib           <- input$n_precalib
    Pxguess              <- as.matrix(Pxguess)
    xguess               <- as.numeric(xguess)
    Pxguesspars          <- as.matrix(Pxguesspars)
    xguesspars           <- as.numeric(xguesspars)
    # estimate the process noise matrix
    Qstates              <- getQ(Qmod[1:numstates], numstates)
    numbeg               <- numstates+1
    numvars              <- numpars+numstates
    Qpars                <- getQ(Qmod[numbeg:numvars], numpars)
    if (is.null(x_pars)==TRUE) {
      # store the random samples in x_pars
      x_pars               <- array(NA, c(n_precalib, numvars))
      # combined state parameter vector
      xpar                 <- c(input$xguess,input$xguesspars)
      Pxpar                <- diag(c(input$Pxguess,diag(input$Pxguesspars)))
      # Generate random parameter samples
      if (input$prior == 0){
        temp               <- multirandnorm(xpar,Pxpar,n_precalib,input$seed[2])
        # assign
        x_pars             <- temp
        rm(temp)
      } else { 
        location           <- rep(NA,numvars)
        scale              <- rep(NA,numvars)
        for (i in 1:numvars) {
          tempmean         <- xpar[i]
          tempsd           <- Pxpar[i,i]
          location[i]      <- log(tempmean/sqrt(1+tempsd/tempmean^2))
          scale[i]         <- sqrt(log(1+tempsd/tempmean^2))
        }
        scale              <- diag(scale)
        temp               <- multirandnorm(location,scale,n_precalib,input$seed[2])
        # assign  
        x_pars             <- exp(temp)
        rm(temp)
      }    
      # remove negative parameters and states 
      for (i in 1:numvars) {
        if (i==1) {
          temp1             <- which(x_pars[,i]<0)
        } else {
          temp1             <- union(temp1, which(x_pars[,i]<0))
        }
        # b parameter cannot be greater than 1
        if (i==2) {
          temp2           <- which(x_pars[,i]>1)
        } else if (i==3) {
          # q parameter between 2 and 8
          temp3           <- which(x_pars[,i]<2)
          temp4           <- which(x_pars[,i]>8)
        }
      }
      temp                    <- union(union(union(temp1,temp2),temp3),temp4)
      if (length(temp)>0) {
        x_pars                <- x_pars[-temp,]        
        n_precalib            <- dim(x_pars)[1]             
        rm(location,scale,temp1,temp2,temp3,temp4,temp)
      }
    }
    # run the model with these parameters    
    # initialize variables
    count                <- 0
    timerange            <- seq(input$tbeg,predendtime,by=timestep)        # define the timerange to run the model
    allN                 <- length(timerange)                              # total number of time steps
    obsN                 <- length(seq(input$tbeg,obsendtime,by=timestep)) # total number of observations
    xpredall             <- array(NA,dim=c(numvars,allN,n_precalib)) 
    metric               <- array(NA,dim=c(obsN,n_precalib))
    for (i in 1:allN) {
      # Calculate state vector over time from dynamics.      
      # running base model
      for (sample in 1:n_precalib) {      
        xpredall[,1,sample]                 <- x_pars[sample,]                        
        if (i==1) {
          xpredall[1:numstates,i,sample]    <- addprocessnoise(model_handle,Qstates,input$qtype,input$seed[2]+i,xpredall[1:numstates,i,sample],xpredall[numbeg:numvars,i,sample],timestep,i) + input$Bmat*input$uinput[i]      
          xpredall[numbeg:numvars,i,sample] <- xpredall[numbeg:numvars,i,sample]   
        } else { 
          xpredall[1:numstates,i,sample]    <- addprocessnoise(model_handle,Qstates,input$qtype,input$seed[2]+i,xpredall[1:numstates,i-1,sample],xpredall[numbeg:numvars,i-1,sample],timestep,i) + input$Bmat*input$uinput[i]      
          xpredall[numbeg:numvars,i,sample] <- xpredall[numbeg:numvars,i-1,sample]   
        }
        if (i<=obsN) {
          ypred             <- xpredall[1,1:i,sample]            
          metric[i,sample]  <- sum(abs(ypred-setup$z[1:i]))/(i*2*sqrt(R))              
          if (is.nan(metric[i,sample])==TRUE || is.na(metric[i,sample])==TRUE) {metric[i,sample]=1000} 
        }
      }
    }
    # now estimate the precalibration metric, analysis and forecast arrays
    x_mean           <- array(NA,c(numvars,allN))
    x_for_mean       <- array(NA,c(numvars,allN))
    x_cov            <- array(NA,c(numvars,allN))
    x_for_cov        <- array(NA,c(numvars,allN))    
    xpar             <- array(list(),allN)
    for (yr in 1:allN) {      
      if (yr<=obsN) {
        # if metric is less than 1, accept the parameter otherwise reject it
        acceptid   <- which(metric[yr,]<=1)
        temp           <- array(NA,c(numvars,length(acceptid)))
        temp[1,]       <- xpredall[1,allN,acceptid]
        temp[2:3,]     <- xpredall[2:3,yr,acceptid]
        xpar[[yr]]     <- temp      
        foryr      <- yr+1
        for (vars in 1:numvars) {
          x_mean[vars,yr]          <- mean(xpredall[vars,yr,acceptid])
          x_cov[vars,yr]           <- var(xpredall[vars,yr,acceptid])
          if (yr==1) {
            x_for_mean[vars,yr]    <- mean(xpredall[vars,yr,])
            x_for_cov[vars,yr]     <- var(xpredall[vars,yr,])
            x_for_mean[vars,foryr] <- mean(xpredall[vars,foryr,acceptid])
            x_for_cov[vars,foryr]  <- var(xpredall[vars,foryr,acceptid])
          } else if (yr>1 && yr<input$tend && foryr<=allN) {                  
            x_for_mean[vars,foryr] <- mean(xpredall[vars,foryr,acceptid])
            x_for_cov[vars,foryr]  <- var(xpredall[vars,foryr,acceptid])
          }
        }
        if (yr<obsN) {rm(acceptid)}
      } else {
        x_mean[vars,yr]          <- mean(xpredall[vars,yr,acceptid])
        x_cov[vars,yr]           <- var(xpredall[vars,yr,acceptid])
        x_for_mean[vars,yr]      <- x_mean[vars,yr]
        x_for_cov[vars,yr]       <- x_cov[vars,yr]   
        temp                     <- array(NA,c(numvars,length(acceptid)))
        temp[1,]                 <- xpredall[1,allN,acceptid]
        temp[2:3,]               <- xpredall[2:3,yr,acceptid]
        xpar[[yr]]               <- temp 
      }
    }
    precout  <- list(x_mean=x_mean,x_cov=x_cov,x_for_mean=x_for_mean,x_for_cov=x_for_cov,xpar=xpar)    
    rm(xpredall,x_pars,x_mean,x_cov,x_for_mean,x_for_cov,xpar)
    # ending time counter
    end.time          <- Sys.time()
    precout$timetaken <- end.time - start.time
    if (input$timeflag == 1) {print(paste("Function time is ",round(as.numeric(precout$timetaken,units="secs"),2),"seconds"))}    
    return(precout)  
  }