#' @name runmcmc
#' @aliases runmcmc
#' @title Run MCMC on any of the test cases
#' @description This function runs MCMC on the test case
#' @param logposterior The posterior density calculation function
#' @param input The problem's input including the values for measurement nosie, process noise, etc.
#' @details %%  ~~ If necessary, more details than the dcription above ~~
#' @return  Mean values of the variable vector from the last 50% of the chain
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @seealso \code{\link{basecasesetup}} \code{\link{setbasecasedefault}}
#' @examples %%  ~~any examples~~
#' @importFrom mcmc metrop
#' @export

runmcmc <-
  function(logposterior,input,setup,calendtime=input$tend,predendtime=input$tend,option=0) {
    # starting time counter
    start.time    <- Sys.time()
    model_handle  <- input$model_handle
    timestep      <- input$timestep
    Qmod          <- input$Qmod
    nummcmc       <- input$nummcmc
    numstates     <- input$numstates
    numpars       <- input$numpars
    qtype         <- input$qtype
    seed          <- input$seed
    Bmat          <- input$Bmat
    uinput        <- input$uinput
    numbeg        <- numstates+1
    numvars       <- numpars+numstates    
    Qstates       <- getQ(Qmod[1:numstates], numstates)
    Qpars         <- getQ(Qmod[numbeg:numvars], numpars)
    xguess        <- c(input$xguess,input$xguesspars)
    Pxguess       <- c(input$Pxguess,diag(input$Pxguesspars))
    numtimes      <- length(seq(1,predendtime,timestep))
    # run the metropolis (vanilla mcmc) algorithm on the problem using all observations from time = 1 to time marked by calendtime
    x_for_mean    <- array(NA,c(numvars,input$tend))      
    x_for_cov     <- array(NA,c(numvars,input$tend))
    x_ana_mean    <- array(NA,c(numvars,input$tend))      
    x_ana_cov     <- array(NA,c(numvars,input$tend))
    outall        <- array(list(),numtimes)
    for (runtime in 1:calendtime) {
      # run mcmc using observations from 1:calendtime
      metropout   <- metrop(logposterior,xguess,nummcmc,scale=Pxguess,input=input,yobs=setup$z[1:runtime],
                                         xstate=NULL,begintime=1,endtime=runtime,option=option)
      beg         <- ceiling(dim(metropout$batch)[1]/2)
      end         <- dim(metropout$batch)[1]      
      numchain    <- length(seq(beg,end,1))
      outall[[runtime]]        <- array(NA,c(numvars,numchain))      
      outall[[runtime]]        <- t(metropout$batch[beg:end,])
    }      
    # now, run the model with the chain outputs at each time step  
    allchainana                <- array(NA,dim=c(numvars,calendtime,numchain))
    allchainfor                <- array(NA,dim=c(numvars,calendtime,numchain))
    for (yrs in 1:calendtime) {
      if (yrs < input$tend) {
        endtime                <- yrs+1
      } else {
        endtime                <- yrs
      }
      for (chain in 1:numchain)  {
        temp                               <- array(NA,dim=c(numvars,endtime))    
        temp[,1]                           <- outall[[yrs]][,chain]
        for (runtime in 1:endtime) {
          if (runtime==1) {
            temp[1:numstates,runtime]      <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+runtime,temp[1:numstates,runtime],temp[numbeg:numvars,runtime],timestep,runtime)+Bmat*uinput[runtime]          
          } else {
            temp[1:numstates,runtime]      <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+runtime,temp[1:numstates,runtime-1],temp[numbeg:numvars,runtime-1],timestep,runtime)+Bmat*uinput[runtime]        
            temp[numbeg:numvars,runtime]   <- temp[numbeg:numvars,runtime-1]
          }
        }
        tempyrs                            <- yrs+1
        if (yrs==1) {
          allchainfor[1:numstates,yrs,chain]   <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+yrs,xguess[1:numstates],xguess[numbeg:numvars],timestep,yrs)+Bmat*uinput[yrs]          
          allchainfor[numbeg:numvars,yrs,chain]<- xguess[numbeg:numvars]
          allchainana[,yrs,chain]              <- temp[,endtime-1]
          if (yrs<calendtime) {
            allchainfor[,tempyrs,chain]        <- temp[,endtime]    
          }
        } else if (yrs == input$tend) {
          allchainana[,yrs,chain]              <- temp[,endtime]
        } else {
          allchainana[,yrs,chain]              <- temp[,endtime-1]
          if (yrs<calendtime) {
            allchainfor[,tempyrs,chain]        <- temp[,endtime]       
          }
        }
        rm(temp)
      }
    }
    # assign forecast and analysis vectors
    if (calendtime < input$tend) {
      x_for_mean[,1:calendtime]         <- apply(allchainfor,c(1,2),mean)      
      x_for_cov[,1:calendtime]          <- apply(allchainfor,c(1,2),var)       
      x_ana_mean[,1:calendtime]         <- apply(allchainana,c(1,2),mean)      
      x_ana_cov[,1:calendtime]          <- apply(allchainana,c(1,2),var)            
    } else {
      x_for_mean                        <- apply(allchainfor,c(1,2),mean)
      x_for_cov                         <- apply(allchainfor,c(1,2),var)        
      x_ana_mean                        <- apply(allchainana,c(1,2),mean)
      x_ana_cov                         <- apply(allchainana,c(1,2),var)        
    }    
    # assign mean and variance of analysis at each time step
    mcmctrue                            <- array(NA,dim=c(numvars,numtimes))
    if (calendtime < predendtime) {
      starttime                         <- calendtime + 1
      # run the model from calendtime + 1 to input$tend to find the long term phosphorus concentration at t=input$tend
      outlast                           <- array(NA,dim=c(numvars,numtimes,numchain))
      outlast[,1:calendtime,]           <- allchainana
      tempN                             <- length(seq(starttime:predendtime))
      for (chain in 1:numchain)  {
        temp                            <- array(NA,dim=c(numvars,tempN))    
        temp[,1]                        <- allchainana[,calendtime,chain]
        for (runtime in starttime:predendtime) {
          tempi     <- runtime - starttime + 1
          if (runtime==starttime) {
            temp[1:numstates,tempi]     <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+runtime,temp[1:numstates,tempi],temp[numbeg:numvars,tempi],timestep,runtime)+Bmat*uinput[runtime]          
          } else {
            temp[1:numstates,tempi]     <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+runtime,temp[1:numstates,tempi-1],temp[numbeg:numvars,tempi-1],timestep,runtime)+Bmat*uinput[runtime]        
            temp[numbeg:numvars,tempi]  <- temp[numbeg:numvars,tempi-1]
          }
          outlast[,runtime,chain]       <- temp[,tempi] 
        }
        rm(temp)
      }
      x_ana_mean                         <- apply(outlast,c(1,2),mean)
      x_ana_co                           <- apply(outlast,c(1,2),var)
      x_for_mean[,starttime:predendtime] <- x_ana_mean[,starttime:predendtime]
      x_for_cov[,starttime:predendtime]  <- x_ana_cov[,starttime:predendtime]
    } else {
      outlast                            <- allchainana         
    }    
    # assign output      
    mcmcout  <- list(x_mean=x_ana_mean,x_cov=x_ana_cov,x_for_mean=x_for_mean,outlast=outlast,outall=outall)    
    rm(x_mean,x_cov,x_for_mean,x_for_cov,outlast,outall)
    # ending time counter
    end.time   <- Sys.time()
    mcmcout$timetaken <- end.time - start.time
    if (input$timeflag == 1) {print(paste("Function time is " , round(as.numeric(mcmcout$timetaken, units="secs"),2), "seconds"))}        
    return(mcmcout)    
  }