#' @name setlakedefault
#' @aliases setlakedefault
#' @title Set the default values of the lake data assimilation problem
#' @description This function sets the default values of the lake input parameters such as parameter q, input poolution vector, the number of filters to run, etc.
#' @return  A list with the default values of the demostration example
#' @details %%  ~~ If necessary, more details than the dcription above ~~
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @seealso \code{\link{lakesetup}}, \code{\link{f_lakestate}}, \code{\link{f_lakepars}}, \code{\link{h_lake}}
#' @examples
#' #using the demostration example set up
#' input <- setlakedefault()
#' @export

setlakedefault <- function(seed = NULL, option = 1)
{
  #initialize to default
  default                    <- list()
  #set the name if this initialization
  default$name               <- 'lakemodel'
  #choose the filters to try: 1="on", 0="off"
  default$runKF              <- 1
  default$runEKF             <- 1
  default$runEnKF            <- 1
  default$runPF              <- 1
  default$prior              <- 1 # 0 for normal prior and 1 for lognormal
  # choose measurement variance
  # in reality, this would be set by the measurement device  
  default$R                  <- 0.01
  # choose model stochastic noise
  # note: particle filter will collapse to a single point very quickly in case of small process noise 
  # one can set it to some small non zero by multiplying Qmod with 1
  default$Qmod               <- c(0.01, 0.01, 0.01)*0
  default$qtype              <- 0 # 0 for normal and 1 for lognormal noise
  # choose beginging and end time for the simulation (years)
  default$tbeg               <- 1
  default$tend               <- 100  
  #choose time interval
  default$timestep           <- 1
  #choose the seed type
  if (is.null(seed) == TRUE) {
   default$seed              <- c(as.numeric(Sys.time()),as.numeric(Sys.time()))
  } else if (length(seed)==1) {
    default$seed             <- c(seed, as.numeric(Sys.time()))
  } else {
    default$seed             <- seed
  }
  #setting up filters
  default$n_ens              <- 50000
  default$resamplechoice     <- 2
  default$n_par              <- default$n_ens   # for both filters to be comparable
  default$plottype           <- 1
  # setting up the truth, first guess, initial esitmate of covariance
  default$xtruestate         <- array(c(0.05)     ,dim=c(1,1))            # true phosphorus in the lake (-), b parameter (-), q parameter (-)
  default$xguess             <- array(c(0.20)     ,dim=c(1,1))            # first guess for the state variables and parameters
  default$Pxguess            <- 0.20                                      # initial estimate of state and parameter error covariance
  default$xtruepars          <- array(c(0.42, 2.00) ,dim=c(2,1))          # true phosphorus in the lake (-), b parameter (-), q parameter (-)
  default$xguesspars         <- array(c(0.62, 4.00) ,dim=c(2,1))          # first guess for the state variables and parameters
  default$Pxguesspars        <- diag(c(0.20, 1.00))                       # initial estimate of state and parameter error covariance
  # define the number of iterations for performance calculations
  default$numtimes           <- 10
  # define the number of samples for precalibration
  default$n_precalib         <- default$n_ens
  # define the number of times to run mcmc
  default$mcmcmult           <- (default$tend*(default$tend+1))/(2*default$tend)
  default$nummcmc            <- ceiling(default$n_ens/default$mcmcmult)
  # read in the default anthropogenic pollution load time series
  if (option==0) {
    filename                 <- system.file("extdata", "noneutrophic.txt", package = "learningmodule")
  } else if (option==1) {
    filename                 <- system.file("extdata", "eutrophic.txt", package = "learningmodule")
  } else if (option==2) {
    filename                 <- system.file("extdata", "eutrophicLin.txt", package = "learningmodule")
  } else if (option==3) {
    filename                 <- system.file("extdata", "eutrophicStep1.txt", package = "learningmodule")
  } else if (option==4) {
    filename                 <- system.file("extdata", "eutrophicStep2.txt", package = "learningmodule")
  } else if (option==5) {
    filename                 <- system.file("extdata", "eutrophicLowPhStep.txt", package = "learningmodule")  
  }
  atseries                   <- read.table(filename)
  default$uinput             <- unlist(atseries)
  # set the default mulitplier on the anthropogenic pollution input
  default$Bmat               <- c(1)
  # set the flag on CPU time: 1 (display for each filter), 0  (do not display)
  default$timeflag           <- 1
  default$numpars            <- 2
  default$numstates          <- 1
  default$model_handle       <- f_lakestate
  default$measurement_handle <- h_lakestate
  default$param_handle       <- f_lakepars
  default$parammeas_handle   <- hmat_lakepars
  default$logposteriormod    <- logposteriorlake
  # set labels for plotting
  default$labels             <- c('Phosphorus', 'b parameter', 'q parameter')
  return(default)
}