#' @name threspredict
#' @aliases threspredict 
#' @title Assess the threshold crossing time
#' @description This function the threshold crossing time for a given set of initial parameter set(s)
#' @param object  A matrix or vector with initial condition(s)
#' @param truth The true values of the variables (in case of a synthetic experiment or the demostration example)
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  A list with values of both criteria
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso \code{\link{kalmanfitler}} \code{\link{extendedkalmanfitler}} \code{\link{ensemblekalmanfitler}} \code{\link{particlefitler}}
#' @examples  
#' model_handle       <- f_model
#' measurement_handle <- h_meas
#' input              <- setdefault()
#' setup              <- problemsetup(input)
#' kalmanout          <- dafilter(model_handle, measurement_handle, setup, input)
#' kalperf            <- filterperformance(kalmanout, setup$xtrue, mcmcout)
#' exkalperf          <- filterperformance(exkalmanout, setup$xtrue, mcmcout)
#' enkalperf          <- filterperformance(enkalmanout, setup$xtrue, mcmcout)
#' partperf           <- filterperformance(partout, setup$xtrue, mcmcout)
#' precalibperf       <- filterperformance(precalib, setup$xtrue, mcmcout)
#' @export

threspredict <-
  function(object, input,  setup, predyr, thresval)
  {        
    #estimate the size
    numvars          <- length(input$xtrueinit)
    N                <- length(seq(0,input$tend))
    
    # set the prediction times
    timerange        <- seq(0,input$tend, by = input$timestep)         # define the timerange to run the model
    N                <- length(timerange)                            # total number of time steps    
    
    # required prediction times    
    numsim           <- dim(object$xvals)[3]
    
    # set output array size
    outobj           <- array(NA, c(numvars, N, numsim))
    fracval          <- array(NA,  c(N,1))
    # initialize some year counts
    inityr                <- predyr
    siminit               <- inityr + 1           
    # run the model numsim times
    for (itr in 1 : numsim) {         
      temp                  <- object$xvals[,1:inityr,itr]      
      outobj[,1:inityr,itr] <- temp
      rm(temp)      
      # now loop over time
      for (simt in siminit : N) {        
        outobj[,simt,itr]   <-  addprocessnoise(input$model_handle, outobj[,simt-1,itr], input$timestep, input$Qmod, input$seed[2]+simt, simt-1, input$qtype) + input$Bmat*input$uinput[simt-1]                
      }
    }
      # assess what fraction of these simulations are over the threshold
      # in case only one simulation value is given, assess the fraction using covariance matrix
      for (simt in 1 : N) {        
        temp      <- outobj[1,simt,]        
      if (length(temp)==1) {
          covvals  <- object$covmat[1]                
          fracval[simt] <- 1 - pnorm(thresval, mean = temp, sd = covvals)
        } else {
          fracval[simt] <- length(which(temp>=thresval)==TRUE)/length(temp)
        }        
      rm(temp)
      }              
      # find the year at which 80% of the simulations are over the threshold
      if (length(which(fracval>0.80))==0) {
        thresyr <- 101
      } else {
        thresyr <- min(which(fracval>0.95))
      }          
    return(thresyr)      
  }