#' @name tippingdetect
#' @aliases tippingdetect 
#' @title Detect the tipping point for the true parameters.
#' @description The tipping point is the farthest year at which all the emissions can be reduced to zero without tipping the lake over
#' @param input The input object containing the initialized values of the filter's true parameters 
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return The year of tipping
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso \code{\link{lakesetup}} \code{\link{setlakedefault}}
#' @examples  
#' input              <- setlakedefault()
#' setup              <- lakesetup(input)
#' tippingyr          <- tippingdetect(input, setup)
#' @export

tippingdetect <-
  function(input,  setup, tipthres)
  {        
    #estimate the number of variables in the problem    
    model_handle   <- input$model_handle
    timestep       <- input$timestep
    Qmod           <- input$Qmod
    numstates      <- input$numstates
    numpars        <- input$numpars
    qtype          <- input$qtype
    seed           <- input$seed
    Bmat           <- input$Bmat
    uinput         <- input$uinput
    numbeg         <- numstates+1
    numvars        <- numpars+numstates
    Qstates        <- getQ(Qmod[1:numstates], numstates)
    Qpars          <- getQ(Qmod[numbeg:numvars], numpars)
    timerange      <- seq(input$tbeg,input$tend,by=timestep)   # define the timerange to run the model
    N              <- length(timerange)                              # total number of time steps    
    # stop the pollution suddenly from years 1 to 99, find the largest year at which the flip occurs
    numsims        <- input$tend
    xpreds         <- array(NA,dim=c(numvars,N,numsims))
    prediff        <- array(NA, dim=c(numsims))
    numfor         <- length(input$measurement_handle(input$xtruestate))
    # run the model
    for (sims in 1:numsims) {
      xpreds[1:numstates,1,sims]    <- input$xtruestate
      xpreds[numbeg:numvars,1,sims] <- input$xtruepars      
      for (t in 1:input$tend) {
        if (t < sims) {
          if (t==1) {
            xpreds[1:numstates,t,sims]    <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+t,xpreds[1:numstates,t,sims],xpreds[numbeg:numvars,t,sims],timestep,t)+Bmat*uinput[t]
            xpreds[numbeg:numvars,t,sims] <- xpreds[numbeg:numvars,t,sims]              
          } else {
            xpreds[1:numstates,t,sims]    <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+t,xpreds[1:numstates,t-1,sims],xpreds[numbeg:numvars,t-1,sims],timestep,t)+Bmat*uinput[t]
            xpreds[numbeg:numvars,t,sims] <- xpreds[numbeg:numvars,t-1,sims]              
          }
        } else {
          if (t==1) {
            xpreds[1:numstates,t,sims]    <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+t,xpreds[1:numstates,t,sims],xpreds[numbeg:numvars,t,sims],timestep,t)+Bmat*uinput[t]
            xpreds[numbeg:numvars,t,sims] <- xpreds[numbeg:numvars,t,sims]              
          } else {
          xpreds[1:numstates,t,sims]      <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+t,xpreds[1:numstates,t-1,sims],xpreds[numbeg:numvars,t-1,sims],timestep,t)+0
          xpreds[numbeg:numvars,t,sims]   <- xpreds[numbeg:numvars,t-1,sims]  
          }
        }
      }
      # estimate the difference between the first and last year predicted phosphorus
      prediff[sims]  <- xpreds[1,input$tend,sims]-xpreds[1,1,sims]
    }
    # determine the year of flip as the largest year at which prediff is lesser than the tippingthres
    flipyr <- max(which(prediff<tipthres))
    return(flipyr)      
  }